// Libraries
import React, { PureComponent } from 'react'

import PropTypes from 'prop-types'

// Theme
import theme from 'theme'
import { AvatarStyled } from './Avatar.styles'


class Avatar extends PureComponent {
  static defaultProps = {
    size: 'md',
    type: 'round',
    src: null
  }

  static propTypes = {
    name: PropTypes.string.isRequired,
    src: PropTypes.string,
    size: PropTypes.string,
    type: PropTypes.string
  }

  /**
   * The getInitials is responsible by to get the
   * initial from the first and last word given by parent
   * @param {string} name
   * @returns {string}
   * @memberof Avatar
   */
  getInitials = (name) => {
    const initials = name.match(/\b\w/g) || []
    return ((initials.shift() || '') + (initials.pop() || '')).toUpperCase()
  }

  /**
   * The convertStringToHash is responsible to convert a string
   *  to hash number
   *
   * @param {string} val
   * @returns {number}
   * @memberof Avatar
   */
  convertStringToHash = (val) => {
    return Array.from(val.replace(/\s/g, ''))
      // eslint-disable-next-line no-bitwise
      .reduce((s, c) => Math.imul(31, s) + c.charCodeAt(0) | 0, 0)
  }

  /**
   * The convertNameToColor is responsible by to convert the
   * avatar name in to a availabled color
   * @param {string} name
   * @returns {string}
   * @memberof Avatar
   */
  convertNameToColor = (name) => {
    const { colors } = theme.avatar

    // get the hash number from avatar name
    const hashName = this.convertStringToHash(name)
    // map the available color to hash
    const charCodeColorArray = colors.map(item => this.convertStringToHash(item))

    // map the available color to hash as object
    const colorBycharCode = colors.map((item) => {
      const code = this.convertStringToHash(item)
      const obj = {}
      obj[code] = item
      return obj
    })

    // get the closest code hash
    const closestCode = charCodeColorArray.reduce((prev, curr) => {
      return (Math.abs(curr - hashName) < Math.abs(prev - hashName) ? curr : prev)
    })

    // filter the correspondent color
    const color = (colorBycharCode.filter(item => item[closestCode]))[0][closestCode]
    return color
  }

  render() {
    const {
      name,
      src,
      type,
      size
    } = this.props
    const initials = this.getInitials(name)
    const color = this.convertNameToColor(name)

    return (
      <AvatarStyled
        color={color}
        type={type}
        size={size}
      >
        {src ? <img src={src} alt={name} /> : initials}
      </AvatarStyled>
    )
  }
}

export default Avatar
