// Libraries
import React, { PureComponent, Fragment } from 'react'
import PropTypes from 'prop-types'
import styled from 'styled-components'
import { BrowserRouter } from 'react-router-dom'

// Theme
import theme from 'theme'

import Link from '../Link'
import { Icon } from '../Icon'

// Icon style
const BreadcrumbIcon = styled(Icon)`
  color: ${theme.icons.color['100']};
  height: 10px;
  margin-top: 3px;
`

// Container style
const BreadcrumbContainer = styled.div`
  display: flex;
  flex-wrap: wrap;
  align-items: center;
  > *:last-of-type:hover{
      text-decoration: none;
      cursor: default;
  }
`

// Link container style
const LinksContainer = styled.div`
  max-width: ${({ min, max }) => min || min && max ? "40px" : "200px"};
  white-space: nowrap;
  text-overflow: ellipsis;
  overflow: hidden;
  text-align: center;
`

class Breadcrumb extends PureComponent {
  static propTypes = {
    maxWidth: PropTypes.bool,
    minWidth: PropTypes.bool,
    children: PropTypes.oneOfType([
      PropTypes.arrayOf(PropTypes.shape({
        type: PropTypes.oneOf([Link])
      })),
      PropTypes.shape({
        type: PropTypes.oneOf([Link])
      })
    ]).isRequired,
  }

  /**
   * The addBreadcrumbIconBetweenLinks is responsible by to clone
   * the children links and add the BreadcrumbIcon between them
   *
   * @memberof Breadcrumb
   * @param {arrayOf(Link)} children
   * @returns {arrayOf(Link)}
   */
  addBreadcrumbIconBetweenLinks = (children, maxWidth , minWidth) => {
    if (!Array.isArray(children)) return children
    const clonedChildrens = [...children]
    const lastComponent = (clonedChildrens.pop()).props
    const currentLink = (
      <Link key={clonedChildrens.length} {...lastComponent}>
        {!lastComponent.children ? lastComponent.title : lastComponent.children}
      </Link>
    )

    const previousLinks = clonedChildrens.map(({ props }, index) => (
      <Fragment key={index.toString()}>
        { minWidth || maxWidth ? (
          <LinksContainer max={maxWidth} min={minWidth} >
            <Link  {...props}>
              {!props.children ? props.title : props.children}
            </Link>
          </LinksContainer>
        ) : (
          <Link  {...props}>
            {!props.children ? props.title : props.children}
          </Link>
        )}
        <BreadcrumbIcon name="Breadcrumb" />
      </Fragment>
    ))
    return [...previousLinks, currentLink]
  }
  
  render() {
    const {
      children,
      maxWidth,
      minWidth
    } = this.props

    const styledChildren = this.addBreadcrumbIconBetweenLinks(children, maxWidth, minWidth)

    return (
      <BreadcrumbContainer>
        <BrowserRouter>
          {styledChildren}
        </BrowserRouter>
      </BreadcrumbContainer>
    )
  }
}

Breadcrumb.defaultProps = {
  minWidth: false,
  maxWidth: false,
}

export default Breadcrumb
