/* eslint-disable react/prop-types */
import CustomOptionsContainer from 'components/CustomSelect/CustomOption'
import {
  hasSearchValue,
  createOption,
  filterOptions,
  getDefaultIndex, isInputInFocus, keyboardActions,
  setPositionOfDropDown
} from 'components/CustomSelect/utils'
import React, { Component } from 'react'
import PropTypes from 'prop-types'
import Input from '../Input'
import { Icon } from '../Icon'
import ButtonIcon from '../ButtonIcon'
import { Small } from '../Typography'
import Option from '../Option'
import GlobalStyle from '../GlobalStyle'
import Styles, { ErrorMessage } from './CustomSelect.styles'


export default class CustomSelect extends Component {
  static propTypes = {
    open: PropTypes.bool,
    options: PropTypes.arrayOf(PropTypes.oneOfType([
      PropTypes.string,
      PropTypes.shape({
        label: PropTypes.string,
        value: PropTypes.string,
        render: PropTypes.node
      })
    ])).isRequired,
    onChange: PropTypes.func.isRequired,
    required: PropTypes.bool,
    disabled: PropTypes.bool,
    readOnly: PropTypes.bool,
    isSearch: PropTypes.bool,
    searchBy: PropTypes.string,
    placeholder: PropTypes.string,
    style: PropTypes.shape(Object),
    maxHeightDropdown: PropTypes.string,
    closeOnSelection: PropTypes.bool,
    highlightSelection: PropTypes.bool,
    inputText: PropTypes.string,
    defaultValue: PropTypes.oneOfType([
      PropTypes.string,
      PropTypes.number,
      PropTypes.shape({
        label: PropTypes.string,
        value: PropTypes.string,
        render: PropTypes.node
      })
    ]),
    iconLeft: PropTypes.oneOfType([
      PropTypes.arrayOf(PropTypes.shape({
        type: PropTypes.oneOf([Icon, ButtonIcon])
      })),
      PropTypes.shape({
        type: PropTypes.oneOf([Icon, ButtonIcon])
      })
    ]),
    iconRight: PropTypes.oneOfType([
      PropTypes.arrayOf(PropTypes.shape({
        type: PropTypes.oneOf([Icon, ButtonIcon])
      })),
      PropTypes.shape({
        type: PropTypes.oneOf([Icon, ButtonIcon])
      })
    ])
  }

  static defaultProps = {
    open: false,
    required: false,
    readOnly: false,
    disabled: false,
    isSearch: false,
    searchBy: 'value',
    placeholder: '',
    style: {},
    closeOnSelection: true,
    highlightSelection: true,
    inputText: null,
    defaultValue: null,
    iconLeft: null,
    iconRight: (
      <Icon name="CaretDown" size="sm" />
    ),
    maxHeightDropdown: null
  }

  constructor(props) {
    super(props)

    this.state = {
      open: props.open,
      selected: -1,
      virtuallySelected: 0,
      searchedValue: null,
      filteredOptions: []
    }
    this.options = []
    this.nodeRef = React.createRef()
    this.listRef = React.createRef()
    this.timeoutID = null
    this.id = Math.random()
    this.optionJustClicked = false
  }

  componentDidMount = () => {
    const { options } = this.props
    this.options = options.map(createOption)
    this.updatePosition = setPositionOfDropDown(this.nodeRef)
    this.updateSelectedOption()
    this.setState({
      filteredOptions: this.options
    })
  }

  componentWillUnmount = () => {
    this.updatePosition = null
  }

  componentDidUpdate = ({ defaultValue: oldDefaultValue, options: oldOptions }) => {
    const { searchedValue } = this.state
    const {
      isSearch, defaultValue, options, searchBy
    } = this.props

    if (defaultValue !== oldDefaultValue) {
      this.updateSelectedOption()
    }

    if (oldOptions !== options) {
      this.options = options.map(createOption)
      if (!isSearch || !hasSearchValue(searchedValue)) {
        // eslint-disable-next-line react/no-did-update-set-state
        this.setState({
          filteredOptions: this.options
        })
      }

      if (isSearch && hasSearchValue(searchedValue)) {
        // eslint-disable-next-line react/no-did-update-set-state
        this.setState({
          filteredOptions: filterOptions(this.options, searchedValue, searchBy)
        })
      }
    }
  }

  /**
   * The setDefaultValue is responsible to set up
   * the select value when the component was mounted
   *
   * @returns {void}
   * @memberOf CustomSelect
   */
  updateSelectedOption = () => {
    const { defaultValue } = this.props
    const { options } = this

    if (options && options.length) {
      this.setState({
        selected: getDefaultIndex(defaultValue, options),
        virtuallySelected: getDefaultIndex(defaultValue, options)
      })
    }
  }

  /**
   * The handleArrows is responsible by handle
   * the arrow navigation
   *
   * @param {EventListener} e
   * @memberOf CustomSelect
   * @returns {*}
  */
  handleArrows = (event) => {
    const { isSearch, closeOnSelection } = this.props
    const {
      open, virtuallySelected, filteredOptions, searchedValue
    } = this.state

    const keyPress = keyboardActions({
      options: filteredOptions, open, isSearch, searchedValue
    }, this.nodeRef, this.listRef)

    keyPress(
      event,
      virtuallySelected,
      ({ selected: index, submit, open: stayOpen }) => {
        if (submit) {
          if (filteredOptions[index]) {
            this.onSelect(
              filteredOptions[index].index,
              closeOnSelection
            )
          }
        } else {
          if (index > -1) {
            this.onNavigate(index)
          }
          if (stayOpen !== undefined) {
            this.setState({ open: stayOpen })
          }
        }
      }
    )
  }

  onNavigate = (index) => {
    this.setState({
      virtuallySelected: index
    })
  }

  /**
   * The onSelect is triggered when the user
   * choose an option from the dropdown, at the same
   * time this method will replace the value
   * on the input and will send the data to the parent component
   *
   * @param {string} index The index to hbe selected
   * @param close If true it will force close the options
   * @returns {void}
   * @memberOf CustomSelect
   */
  onSelect = (index, close) => {
    const { onChange, closeOnSelection } = this.props
    if (!closeOnSelection) {
      this.optionJustClicked = true
      // wait for onBlurInput to be executed and then reset the flag
      setTimeout(() => {
        this.optionJustClicked = false
      }, 300)
    }
    const { value } = this.options[index]
    const data = { index, value }

    this.focus()

    this.setState({
      selected: index,
      virtuallySelected: index,
      searchedValue: null,
      filteredOptions: this.options,
      open: !close,
    }, () => onChange(data))
  }

  /**
   * Focus the main text input
   *
   * @returns {void}
   * @memberOf CustomSelect
   */
  focus = () => {
    this.nodeRef.current.querySelector('input[type=text]').focus()
  }

  /**
   * Opens the wizard and makes sure the size and position of
   * the options box is set.
   *
   * @returns {void}
   * @memberOf CustomSelect
   */
  onClickInput = () => {
    const { disabled } = this.props
    const { open } = this.state

    if (!isInputInFocus(this.nodeRef)) this.focus()

    this.updatePosition()

    clearTimeout(this.timeoutID)

    if (!disabled) {
      this.setState({
        open: !open
      })
    }
  }

  /**
   * On blur closes the options box with a small delay.
   * No options are selected on blur.
   *
   * @returns {void}
   * @memberOf CustomSelect
   */
  onBlurInput = () => {
    const { closeOnSelection } = this.props
    clearTimeout(this.timeoutID)
    this.timeoutID = setTimeout(() => {
      this.setState({
        // keep dropdown open if closeOnSelection is false and the blur
        // event occur due to clicking on any of the option itself
        open: !closeOnSelection && this.optionJustClicked
      })
      // this delay is important to make selection work before
      // closing the dropdown
    }, 250)
  }

  /**
   * The onSearch works when the parent
   * component pass the prop isSearch as true
   *
   * @param {String} value
   * @returns {void}
   * @memberOf CustomSelect
   */
  onSearch = (value) => {
    // eslint-disable-next-line react/prop-types
    const { isSearch, searchBy } = this.props
    if (isSearch) {
      this.setState({
        searchedValue: value,
        open: true,
        virtuallySelected: 0,
        filteredOptions: filterOptions(this.options, value, searchBy)
      })
    }
  }

  getRealValue = (selected = null) => {
    const { isSearch } = this.props
    const { selected: selectedState, searchedValue } = this.state
    const option = this.options[selected || selectedState]

    if (isSearch && hasSearchValue(searchedValue)) {
      return searchedValue
    }

    if (option) {
      if (typeof option === 'object') {
        return option.label || option.value || ''
      }
    }
    return option || ''
  }

  renderOptions = () => {
    const { virtuallySelected, searchedValue, filteredOptions } = this.state
    // eslint-disable-next-line react/prop-types
    const { isSearch, highlightSelection } = this.props

    if (isSearch && hasSearchValue(searchedValue) && !filteredOptions.length) {
      return <Option size="large">No options</Option>
    }

    return filteredOptions.map(({
      index, group, render, label, value
    }) => {
      return (
        <Option {...{
          key: index,
          index,
          selected: highlightSelection ? virtuallySelected : null,
          onClick: () => (!group ? this.onSelect(index) : null),
          group: group !== undefined
        }}
        >
          {render || label || group || value }
        </Option>
      )
    })
  }

  /**
   * The renderErrorMessage is responsible to render the error message
   *
   * @returns {node}
   * @memberOf CustomSelect
   */
  renderErrorMessage = () => {
    const { required } = this.props
    const value = this.getRealValue()
    return required && value.length === 0
      ? (
        <ErrorMessage>
          <Small color="error"> Please select an option</Small>
        </ErrorMessage>
      )
      : null
  }

  render = () => {
    const { open, searchedValue } = this.state
    const {
      required,
      disabled,
      isSearch,
      readOnly,
      placeholder,
      style,
      iconLeft,
      iconRight,
      maxHeightDropdown,
      inputText
    } = this.props
    return (
      <>
        <Styles.CustomSelectContainer
          style={style}
          ref={this.nodeRef}
        >
          <Styles.InputContainer aria-haspopup="listbox" isSearch={isSearch}>
            <Input
              onClick={() => this.onClickInput()}
              onBlur={() => this.onBlurInput()}
              onChange={val => this.onSearch(val)}
              onKeyDown={e => this.handleArrows(e)}
              iconRight={iconRight}
              iconLeft={iconLeft}
              value={(isSearch && searchedValue) || inputText || this.getRealValue()}
              placeholder={placeholder}
              required={required}
              disabled={disabled}
              readOnly={!isSearch || readOnly}
              isSearch={isSearch}
              large
            />
            { this.renderErrorMessage()}
          </Styles.InputContainer>
          {this.nodeRef.current && (
          <CustomOptionsContainer
            open={open}
            maxHeight={maxHeightDropdown}
            nodeRef={this.nodeRef}
            listRef={this.listRef}
          >
            { this.renderOptions() }
          </CustomOptionsContainer>
          ) }
        </Styles.CustomSelectContainer>
        <GlobalStyle />
      </>
    )
  }
}
