/* eslint-disable import/prefer-default-export */

export const hasSearchValue = (searchedValue) => {
  return typeof (searchedValue) === 'string'
}

/**
 * Extracts the default value based on the type of value
 * provided.
 *
 * For any type of option user can provide any type of
 * default value, meaning, if user provides an array
 * of strings as option, he can still provide an object
 * with a value key as the default and vice versa.
 *
 * @param defaultValue
 * @param options
 * @returns {*}
 */
export const getDefaultIndex = (defaultValue, options) => {
  if (defaultValue === undefined || defaultValue === null) {
    return null
  }

  const type = typeof defaultValue

  switch (type) {
    case 'string':
      return options.findIndex(
        ({ value }) => value === defaultValue
      )
    case 'number':
      return defaultValue >= 0 ? defaultValue : 0
    default:
      return options.findIndex(
        ({ value }) => value === defaultValue.value
      )
  }
}

/**
 * Based on the option value type, normalizes the option
 * into an object that the CustomSelect can understand.
 *
 * @param option A option value as the user has passed in
 * @param index The index of this option
 * @returns {{index: number, value: *}}
 */
export const createOption = (option, index) => {
  if (option === undefined || option === null) {
    return {
      index,
      value: null
    }
  }
  const type = typeof option
  switch (type) {
    case 'string':
    case 'number':
      return {
        index,
        value: option
      }
    default:
      return { ...option, index }
  }
}

/**
 * The getOffsetInputPosition it's responsible by to get
 * the input's position relative to the screen
 *
 * @returns {Object}
 */
export const getOffsetInputPosition = (nodeRef) => {
  try {
    if (nodeRef && nodeRef.current) {
      const input = nodeRef.current.querySelector('input[type=text]')
      const rect = input.getBoundingClientRect()
      return {
        top: rect.top,
        left: rect.left,
        bottom: window.innerHeight - rect.top - rect.height,
        height: rect.height
      }
    }
  } catch (e) {
    return {}
  }
  return {}
}

/**
 * Cauculates the height of the options container based on the
 * distance of the input to the
 * @param nodeRef
 * @returns {number}
 */
export const getOptionsContainerHeight = (nodeRef) => {
  const screenHeight = window.innerHeight
  const rect = getOffsetInputPosition(nodeRef) || {}
  let offSet

  if (rect.top < rect.bottom) {
    offSet = screenHeight && screenHeight > 0 ? screenHeight - rect.top - rect.height : 0
  } else {
    offSet = screenHeight && screenHeight > 0 ? screenHeight - rect.bottom - rect.height : 0
  }

  if (offSet <= 120) {
    return 100
  }

  return offSet - 20
}

/**
 * Calculates the options container height based on the distance
 * the input is from the top and bottom of the window.
 *
 * If the distance from the top is bigger than from the bottom,
 * the options will open upwards, same if the distance from the
 * bottom of the window is bigger, it will open downwards.
 *
 * @returns {function}
 */
export const setPositionOfDropDown = (nodeRef) => {
  return () => {
    const position = getOffsetInputPosition(nodeRef)
    const { top, bottom } = position

    if (top > bottom) {
      nodeRef.current.classList.add('to-top')
    } else {
      nodeRef.current.classList.remove('to-top')
    }
  }
}

export const isInputInFocus = (nodeRef) => {
  const activeItem = document.activeElement.nodeName
  const input = nodeRef.current.querySelector(`:scope ${activeItem.toLowerCase()}[type=text]`)
  return (input && activeItem === 'INPUT')
}

/**
 * Options filter
 *
 * Search is currently handled by first checking which options startsWith
 * the search string, if no records are found, than we try a fuzzier search by
 * using the contains method.
 *
 * @param options
 * @param searchString
 * @param searchBy
 * @returns {*}
 */
export const filterOptions = (options, searchString, searchBy) => {
  const searchProp = searchBy || 'value'
  // some options are not real options, they're just group labels
  // so let's extract the real options
  const optionsToFilter = options.filter(option => option.value)

  const filterOptionByStartWith = option => option[searchProp]
    .toString().toLowerCase().startsWith(searchString.toLowerCase())

  const filterOptionByContains = option => option[searchProp]
      .toString().toLowerCase().includes(searchString.toLowerCase())

  if (searchString) {
    const filteredValues = optionsToFilter.filter(filterOptionByStartWith)
    return filteredValues.length ? filteredValues : optionsToFilter.filter(filterOptionByContains)
  }

  return options
}

/**
 * A memoizer that handles up and down arrow keys navigating the options.
 *
 * The returned function will take a key press event and based on the
 * key pressed perform one of the following:
 *
 * - Arrow Up: Move the selected index up in the options
 * - Arrow Down: Move the selected index down in the options
 * - Enter: Set the current focused option as as the value
 * - Esc: Close the options box
 *
 * The provided callback will be called with the state to be updated.
 *
 * @param options
 * @param open
 * @param nodeRef
 * @param listRef
 * @returns {function(...[*]=)}
 */
export const keyboardActions = ({
  options, open
}, nodeRef, listRef) => {
  const scrollTo = (index, callback) => {
    const scrollHeight = parseInt(listRef.current.scrollHeight, 10)
    const itemHeight = parseInt(scrollHeight / options.length, 10)
    const scrollY = (index * itemHeight)
    listRef.current.scrollTo(0, scrollY)
    callback({ selected: index })
  }

  const keyUp = (selected, callback) => {
    if (selected > 0) {
      const newIndex = selected - 1
      scrollTo(newIndex, callback)
    }
  }

  const keyDown = (selected, callback) => {
    const lastIndex = options.length - 1
    if (selected <= lastIndex) {
      /**
       * If the current selected value is the same as the last index, either we
       * are in the last item or our list now only contain one option.
       * If so we send the same index back instead of incrementing so we don't
       * overflow.
       */
      const newIndex = lastIndex === selected ? lastIndex : selected + 1
      scrollTo(newIndex, callback)
    }
  }

  const resolve = (event) => {
    const { keyCode } = event
    switch (keyCode) {
      case 38: // up arrow
        return 'up'
      case 40: // down arrow
        return 'down'
      case 13: // enter key
        return 'enter'
      case 27: // esc key
        return 'esc'
      default:
        return null
    }
  }

  return (event, selected, callback) => {
    const direction = resolve(event)
    const isInputFocus = isInputInFocus(nodeRef)

    if (!isInputFocus) return null

    if (open && direction === 'esc') {
      return callback({ open: false })
    }

    if (!open && isInputFocus && (direction === 'up' || direction === 'down')) {
      return callback({ open: true })
    }

    switch (direction) {
      case 'enter':
        if (selected < 0) return callback({ open: false })
        return callback({ selected, submit: true })
      case 'down':
        return keyDown(selected, callback)
      case 'up':
        return keyUp(selected, callback)
      default:
        return null
    }
  }
}
