/* eslint-disable import/no-extraneous-dependencies */

// Libraries
import React from 'react'
import { storiesOf, addDecorator } from '@storybook/react'
import { withA11y } from '@storybook/addon-a11y'
import { withKnobs, boolean, number } from '@storybook/addon-knobs'

// Components
import EventsLog from './index'
import { ButtonLink, Icon, Typography } from '..'

// Mock data

import {
  emptyEventsLogData,
  irpEventsLogData,
  actionEventsLogData,
  insightsEventsLogData
} from './mock'
import { BrowserRouter } from 'react-router-dom'
import { StoryContainer, ExportWrapper } from './EventsLog.styles'
import { withState } from '@dump247/storybook-state'
import { getColumns } from './utils'

const { H1 } = Typography

addDecorator(withA11y)
addDecorator(withKnobs)

const state = {
  searchValue: '',
  appliedFilters: []
}

const excludeSortColumn = ['action', 'data_source', 'event']
const showExternalFunctionalityAlert = () =>
  console('This functionality will be handled by client using it.')

export const getEportData = () => (
  <ExportWrapper onClick={() => console('Export functionality will be passed down by client')}>
    <Icon name="Download" size="xsm" fill="#0C7D84" />
    <ButtonLink>Export Data</ButtonLink>
  </ExportWrapper>
)

const renderEventsLog = (title, props) => (
  <BrowserRouter>
    <StoryContainer>
      <H1>{title}</H1>
      <EventsLog
        loading={false}
        totalCount={number('Total count', props.data.total)}
        exportData={getEportData()}
        excludeSortColumn={excludeSortColumn}
        shouldDisableExport={boolean('Should disable export', false)}
        onSortedChange={showExternalFunctionalityAlert}
        enableAdditionalDetails={boolean('Enable additional details', false)}
        onChangeTable={showExternalFunctionalityAlert}
        disableSearch={boolean('Should disable search', false)}
        disableFilter={boolean('Should disable additional filters', false)}
        {...props}
      />
    </StoryContainer>
  </BrowserRouter>
)

const renderEventsLogWithState = (title, props) =>
  withState(state)(({ store }) => {
    const handleFilterSelection = selection => {
      if (selection.key === 'date') {
        store.set({
          appliedFilters: [
            {
              key: selection.key,
              value: selection.value,
              original: selection.value
            }
          ]
        })
      }
      const newValue = selection.value.toLowerCase().replace(/\s/g, '_')
      if (selection.checked) {
        store.set({
          appliedFilters: [
            {
              key: selection.key,
              value: newValue,
              original: selection.original
            }
          ]
        })
      } else {
        store.set({
          appliedFilters: store.state.appliedFilters.filter(item => item.key !== selection.key)
        })
      }
    }

    const handleFilterRemoval = selection => {
      store.set({
        appliedFilters: store.state.appliedFilters.filter(
          item => item.key !== selection.key && item.original !== selection.original
        )
      })
    }

    const clearFilter = () =>
      store.set({
        appliedFilters: []
      })

    return renderEventsLog(title, {
      searchValue: store.state.searchValue,
      appliedFilters: store.state.appliedFilters,
      handleSearch: value => store.set({ searchValue: value }),
      clearSearch: () => store.set({ searchValue: '' }),
      onFilterSelection: handleFilterSelection,
      clearFilter: clearFilter,
      onFilterRemoval: handleFilterRemoval,
      ...props
    })
  })

const columns = getColumns('date desc', false, true, true)

storiesOf('EventsLog', module)
  .addParameters({
    jest: ['EventsLog']
  })
  .add('Empty events log', () =>
    renderEventsLogWithState('Empty events log', {
      data: emptyEventsLogData,
      columns
    })()
  )
  .add(
    'Events log on IRP',
    renderEventsLogWithState('Events log on individual risk profile page', {
      data: irpEventsLogData,
      columns
    })
  )
  .add(
    'Events log on actions latest events modal',
    renderEventsLogWithState('Events log on actions latest events modal', {
      data: actionEventsLogData,
      columns
    })
  )
  .add(
    'Events log on actionable insights page',
    renderEventsLogWithState('Events log on actionable insights page', {
      data: insightsEventsLogData,
      showIndividualColumn: boolean('Show individual column', true),
      columns
    })
  )
