import React, { useState, useMemo } from 'react'
import PropTypes from 'prop-types'
import moment from 'moment'

import {
  Table,
  DragosaurWithUmbrella,
  Tooltip,
  SearchInput,
  MoreFilters,
  AppliedFilters,
  Icon
} from '..'
import {
  EventLogsWrapper,
  EventLogTableContainer,
  TableWrapper,
  DragoWrapper,
  EventLogUrlWrapper,
  EventLogCopyWrapper,
  Toolbar,
  FilterContainer,
  ClearFilters,
  EventLogAdditional,
  EventLogHeader,
  EventLogBodyWrapper,
  AdditionalLogNoData,
  EventLogSeverity,
  EventLogSeverityData,
  EventLogFound
} from './EventsLog.styles'

import {
  getColumns,
  getDummyEventsLog,
  InsightDataFilterOptions,
  normalizeActionName,
  similarEventFoundLink
} from './utils'
import IconCopy from './IconCopy'
import { humanize, capitalize } from 'utils/strings'
import { timeWithTimeZone } from 'utils/dates'

const dummyData = getDummyEventsLog()

const EventsLog = props => {
  const {
    loading,
    data,
    columns,
    enableAdditionalDetails,
    onChangeTable,
    totalCount,
    defaultPage,
    defaultPageSize,
    onSortedChange,
    excludeSortColumn,
    shouldDisableExport,
    exportData,
    disableSearch,
    searchValue,
    handleSearch,
    clearSearch,
    disableFilter,
    appliedFilters,
    onFilterSelection,
    clearFilter,
    onFilterRemoval
  } = props

  const [AdditionalLog, setAdditionalLog] = useState(false)
  const [additionalLogData, setAdditionalLogData] = useState({})
  const [showAnimation, setShowAnimation] = useState(AdditionalLog)
  const [resetRowSelection, setResetRowSelection] = useState(false)
  const [copyToggle, setCopyToggle] = useState(false)
  const [additionalFilter, setAdditionalFilter] = useState('')

  /*
    These are props to support additional details on table component when
    enableAdditionalDetails={true}
  */

  let additionalProps = {}

  if (enableAdditionalDetails) {
    additionalProps = {
      tableHover: enableAdditionalDetails,
      highlightSelected: enableAdditionalDetails,
      resetRowSelection: resetRowSelection,
      onClickRow: (rowData, index) => {
        setCopyToggle(false)
        const normalizedName = normalizeActionName(rowData?.action || '')
        setAdditionalFilter(phishingFilter[normalizedName] || normalizedName)
        setAdditionalLogData(rowData?.details || {})
        if (index === -1) {
          animationHandler(false)
          return
        }
        animationHandler(true)
      }
    }
  }

  const isAdditionalLogEmpty = () => {
    return Object.keys(additionalLogData).length !== 0
  }

  const idFields = ['phishing_campaign_nid', 'phishing_campaign_title']

  const getFormattedValue = (value, field) => {
    if (!idFields.includes(field) && moment(value, [moment.ISO_8601]).isValid()) {
      return timeWithTimeZone({
        date: value
      })
    } else if (field === 'url') {
      return (
        <EventLogUrlWrapper>
          {value.length > 37 ? (
            <div className="couponcode">
              {trancateUrl(value)}
              <span className="coupontooltip">{value}</span>
            </div>
          ) : (
            value
          )}
          <EventLogCopyWrapper
            onClick={() => {
              setCopyToggle(true)
              navigator.clipboard.writeText(value)
            }}
          >
            <span>
              <Tooltip body={'Copy full URL'} readOnly size="xsm">
                <IconCopy fill={copyToggle ? '#0c7d84' : '#959DA8'} />
              </Tooltip>
              <p style={{ display: copyToggle ? 'inline' : 'none' }}>Copied!</p>
            </span>
          </EventLogCopyWrapper>
        </EventLogUrlWrapper>
      )
    }

    if (typeof value === 'string' && value) return capitalize(String(value))
    else if (typeof value === 'boolean') return value ? 'Yes' : 'No'
    return value || 'N/A'
  }

  const additionalFiedls = additionalData => {
    return Object.keys(additionalData).map(field => {
      return (
        <>
          <EventLogSeverity>
            {field !== 'url' ? humanize(field) : humanize(field).toUpperCase()}
          </EventLogSeverity>
          <EventLogSeverityData>
            {getFormattedValue(additionalLogData[field], field)}
          </EventLogSeverityData>
        </>
      )
    })
  }

  const animationClasses = {
    true: 'slide-in',
    false: 'slide-out'
  }

  const phishingFilter = {
    actual_phishing: 'real_world_phishing',
    simulated_phishing: 'phishing'
  }

  const animationHandler = isTrue => {
    setShowAnimation(isTrue)
    if (!isTrue) {
      setTimeout(() => setAdditionalLog(isTrue), 150)
      return
    }
    setAdditionalLog(isTrue)
  }

  const handleChangeTable = async info => {
    setAdditionalLog(false)
    onChangeTable(info)
  }

  const onHandleSearch = value => {
    setAdditionalLog(false)
    handleSearch(value)
  }

  const onSelectMoreFilter = selection => {
    setAdditionalLog(false)
    onFilterSelection(selection)
  }

  const onRemoveFilter = selection => {
    setAdditionalLog(false)
    onFilterRemoval(selection)
  }

  const handleClearSearch = () => {
    setAdditionalLog(false)
    clearSearch()
  }

  const handleClearFilter = () => {
    setAdditionalLog(false)
    clearFilter()
  }

  const tableData = loading ? dummyData : data?.list || []

  return (
    <EventLogsWrapper>
      <Toolbar>
        <FilterContainer>
          {!disableSearch && (
            <SearchInput
              value={searchValue}
              handleSearch={onHandleSearch}
              clearSearch={handleClearSearch}
            />
          )}
          {!disableFilter && (
            <MoreFilters
              filters={appliedFilters || []}
              _options={InsightDataFilterOptions}
              optionParams={[data?.metadata]}
              onSelection={onSelectMoreFilter}
            />
          )}
          {appliedFilters?.length > 0 && (
            <ClearFilters onClick={handleClearFilter}>Clear filters</ClearFilters>
          )}
        </FilterContainer>
        {!shouldDisableExport && <div data-test-id="export-data">{exportData}</div>}
      </Toolbar>
      <AppliedFilters appliedFilters={appliedFilters} handleRemoveFilter={onRemoveFilter} />
      <EventLogTableContainer additionalLog={AdditionalLog}>
        {loading || (data && !!data?.list?.length) ? (
          <TableWrapper
            data-test-id="events-log-table-wrapper"
            columns={columns?.length}
            enableAdditionalDetails={enableAdditionalDetails}
            additionalLog={AdditionalLog}
          >
            <Table
              resizable
              bodyHeight="calc(100vh - 360px)"
              data={tableData}
              columns={columns}
              onChangeTable={handleChangeTable}
              manual
              totalCount={totalCount}
              defaultPage={defaultPage}
              defaultPageSize={defaultPageSize}
              onSortedChange={onSortedChange}
              pageSizeOptions={[10, 25, 50, 100]}
              excludeSortColumn={excludeSortColumn}
              {...additionalProps}
            />
          </TableWrapper>
        ) : (
          <DragoWrapper>
            <DragosaurWithUmbrella message={'There are no insight available'} showTooltip={false} />
          </DragoWrapper>
        )}
        {/* <div ref={filterRef} /> */}
        <EventLogAdditional
          additionalLog={AdditionalLog}
          className={animationClasses[showAnimation]}
        >
          <div
            style={{
              display: 'flex',
              flexDirection: 'row',
              justifyContent: 'space-between'
            }}
          >
            <EventLogHeader>Additional Details</EventLogHeader>
            <span
              onClick={() => {
                setResetRowSelection(!resetRowSelection)
                animationHandler(false)
              }}
            >
              <Icon name="Close" stroke="#959DA8" size="lg" />
            </span>
          </div>
          <EventLogBodyWrapper>
            {isAdditionalLogEmpty() ? (
              <>
                <div>{additionalFiedls(additionalLogData)}</div>
                <EventLogFound
                  onClick={() => {
                    animationHandler(false)
                    setResetRowSelection(!resetRowSelection)
                  }}
                >
                  {similarEventFoundLink(data, additionalFilter)}
                </EventLogFound>
              </>
            ) : (
              <AdditionalLogNoData>No additional event details available…</AdditionalLogNoData>
            )}
          </EventLogBodyWrapper>
        </EventLogAdditional>
      </EventLogTableContainer>
    </EventLogsWrapper>
  )
}

EventsLog.defaultProps = {
  loading: false,
  exportConfig: {
    disable: false
  },
  disableSearch: false,
  searchValue: '',
  handleSearch: () => {
    // This is intentional
  },
  clearSearch: () => {
    // This is intentional
  },
  disableFilter: false,
  eventAppliedFilters: [],
  enableAdditionalDetails: false,
  onSortedChange: () => {
    // This is intentional
  },
  defaultPage: 0,
  defaultPageSize: 25,
  shouldDisableExport: false,
  clearFilter: () => {
    // This is intentional
  },
  appliedFilters: [],
  onFilterSelection: () => {
    // This is intentional
  },
  onFilterRemoval: () => {
    // This is intentional
  }
}

EventsLog.propTypes = {
  loading: PropTypes.bool,
  data: PropTypes.shape({}).isRequired,
  columns: PropTypes.array.isRequired,
  onChangeTable: PropTypes.func.isRequired,
  onSortedChange: PropTypes.func,
  disableSearch: PropTypes.bool,
  searchValue: PropTypes.string,
  handleSearch: PropTypes.func,
  clearSearch: PropTypes.func,
  disableFilter: PropTypes.bool,
  eventAppliedFilters: PropTypes.array,
  filters: PropTypes.arrayOf({}),
  enableAdditionalDetails: PropTypes.bool,
  totalCount: PropTypes.number.isRequired,
  defaultPage: PropTypes.number,
  defaultPageSize: PropTypes.number,
  excludeSortColumn: PropTypes.arrayOf(PropTypes.string),
  shouldDisableExport: PropTypes.bool,
  exportData: PropTypes.node,
  clearFilter: PropTypes.func,
  appliedFilters: PropTypes.array,
  onFilterSelection: PropTypes.func,
  onFilterRemoval: PropTypes.func
}

export default EventsLog
