/* eslint-disable no-useless-escape */
import React from 'react'
import * as d3 from 'd3'
import { formatDate } from './utils/dates'
import * as moment from 'moment'

import { BEHAVIOR_COLORS, setLineMargin, showHorizontalLines, yAxisLabel } from './constants'
import { dashToTitleCase } from './utils/string'
import { LegendLabel, LegendBullet, LegendIcon, Separator, Strong } from './HRSGraph.styles'
import Icon from '../Icon'
import Tooltip from '../Tooltip'

const compareBehaviors = (firstBehavior, secondBehavior, sortAlphabetical = false) => {
  const [firstBehaviorName, firstBehaviorScore] = firstBehavior
  const [secondBehaviorName, secondBehaviorScore] = secondBehavior
  // 1. overall should be the first
  if (firstBehaviorName === 'overall' || firstBehaviorName === 'human_risk_score') return -1
  // 2. sorting by score
  if (secondBehaviorScore !== firstBehaviorScore && !sortAlphabetical)
    return secondBehaviorScore - firstBehaviorScore
  // 3. sorting alphabetically
  const firstName = firstBehaviorName === 'phishing' ? 'simulated-phishing' : firstBehaviorName
  const secondName = secondBehaviorName === 'phishing' ? 'simulated-phishing' : secondBehaviorName
  return firstName.localeCompare(secondName)
}

const reduceToMostRecentBehaviorScore = (acc, [key, scores]) => {
  const [mostRecentScore] = [...scores].reverse()
  return [...acc, [key, mostRecentScore?.score || 0]]
}

const sortBehaviors = (behaviorScores, sortAlphabetical = false) =>
  Object.entries(behaviorScores || {})
    .reduce(reduceToMostRecentBehaviorScore, [])
    .sort((first, second) => compareBehaviors(first, second, sortAlphabetical))
    .map(([behaviorName]) => behaviorName)

const getBehaviorLabel = (key, isInsight = false) => {
  const titledCase = dashToTitleCase(key)
  if (!isInsight && titledCase === 'Phishing') return 'Simulated Phishing'
  return titledCase
}

export const getLinesInfoToDrawChart = (
  data,
  keyToBeDashed = 'overall',
  sortAlphabetical = false,
  isInsight = false
) => {
  const tempData = isInsight ? Object.keys(data) : sortBehaviors(data, sortAlphabetical)

  return tempData.reduce(
    (info, key, idx) => ({
      ...info,
      [key]: {
        color: BEHAVIOR_COLORS[idx],
        opacity: key === keyToBeDashed ? 1 : 0.3,
        isDashed: key === keyToBeDashed,
        label: getBehaviorLabel(key, isInsight)
      }
    }),
    0
  )
}

function convertYearToQuarters(data) {
  if (!data || !Object.keys(data).length) return []

  const firstBehavior = data[Object.keys(data)[0]]
  const datesWithQuarters = firstBehavior
    .map(({ date }) => ({
      quarter: moment(date, 'YYYY-MM-DD').format('[Q]Q Y'),
      date
    }))
    .reverse()

  // keep only latest month for each quarter
  const quarters = datesWithQuarters
    .filter(
      (obj, index) =>
        datesWithQuarters.findIndex(({ quarter }) => quarter === obj.quarter) === index
    )
    .reverse()

  const filterByQuarter = behaviourKey => {
    if (!behaviourKey || !quarters || !Array.isArray(quarters)) return []
    return quarters.map(({ date, quarter }, index) => {
      return {
        ...(data[behaviourKey].find(item => item.date === date) || {
          score: null,
          date
        }),
        quarter,
        ...(index > 0
          ? {
              score_last_quarter:
                data[behaviourKey].find(item => item.date === quarters[index - 1].date)?.score ||
                null
            }
          : {
              score_last_quarter: null
            })
      }
    })
  }

  return Object.keys(data).reduce((value, key) => {
    return { ...value, [key]: filterByQuarter(key) }
  }, {})
}

function parseXAxis({ date, quarter }, typeGraph) {
  if (typeGraph === 'monthly') return date
  return quarter
}

export function createChartBar({
  element,
  behaviorScores,
  campaignScores,
  chartReady,
  disabledLines,
  typeGraph = 'monthly',
  onMouseEnter,
  hideActionsTaken,
  printing,
  keyToBeDashed,
  minMaxValue,
  sortAlphabetical = false,
  isInsight,
  yAxisDecimal = false,
  changeLineFocus,
  performanceFlag,
  showGradeLabel = false
}) {
  const parentNode = element.parentElement
  const getSanitizedId = (key, date, score) =>
    `${key}-${date}-${score}${printing ? '-printable' : ''}`.replace(/[.]/g, '-')

  // get parent div sizes
  const styles = window.getComputedStyle(parentNode)
  const padding = parseFloat(styles.paddingLeft) + parseFloat(styles.paddingRight)

  const parentWidth = parentNode.clientWidth - padding || 500
  const parentHeight = 310

  const paddingLeft = showGradeLabel ? 90 : 57
  const paddingTop = showGradeLabel ? 50 : 20

  const svgWidth = parentWidth - paddingLeft
  const svgHeight = parentHeight - paddingTop

  // create container
  const svgContainer = d3
    .select(element)
    .attr('width', '100%')
    .attr('height', parentHeight)

  // create the graph area
  const graphArea = svgContainer
    .append('g')
    .attr('width', '100%')
    .attr('height', '100%')

  const { min, max } = minMaxValue

  const yScale = d3
    .scaleLinear()
    .domain([min, max])
    .rangeRound([svgHeight - paddingTop, 0])

  const yAxis = yAxisLabel({ yScale, showGradeLabel, yAxisDecimal, max })

  // rendering yAxis
  graphArea
    .append('g')
    .style('font-size', '14px')
    .style('line-height', '24px')
    .attr('transform', `translate(${paddingLeft},${paddingTop * 0.5})`)
    .style('cursor', 'pointer')
    .style('font-family', 'Neue Haas Unica Pro')
    .call(yAxis)
    .selectAll('path')
    .attr('class', 'y-axis-line')
    .attr('fill', 'none')
    .attr('stroke', 'none')

  // create xAxis
  const dataGraph = typeGraph === 'monthly' ? behaviorScores : convertYearToQuarters(behaviorScores)
  const firstItemOfObjects = dataGraph[Object.keys(dataGraph)[0]]

  const xScale = d3
    .scaleBand()
    .domain(firstItemOfObjects.map(item => parseXAxis(item, typeGraph)))
    .rangeRound([0, svgWidth])

  const intervention_Scale =
    behaviorScores?.historical_data
      ?.map(historicalData => historicalData.date)
      ?.sort((a, b) => (a > b ? 1 : -1)) || []

  // Exclusive scale for intervention lines
  const xScale2 = d3
    .scaleTime()
    .domain([
      new Date(intervention_Scale[0]),
      new Date(intervention_Scale[intervention_Scale.length - 1])
    ])
    .range([0, ((svgWidth - 6) * 11) / 12])

  const xAxis = d3
    .axisBottom(xScale)
    .tickSize(0)
    // .ticks(d3.timeMonth.every(1))
    .tickPadding(15)
    .tickSizeOuter(0)
    .tickFormat(v => (typeGraph === 'monthly' ? formatDate({ date: v, format: 'MMM' }) : v))
    .scale(xScale)

  // rendering xAxis
  const xAxisRender = graphArea
    .append('g')
    .style('font-size', '14px')
    .style('line-height', '24px')
    .style('cursor', 'pointer')
    .style('font-family', 'Neue Haas Unica Pro')
    .attr('transform', `translate(${paddingLeft},${svgHeight - paddingTop / 2})`)

  xAxisRender
    .call(xAxis)
    .selectAll('path')
    .attr('fill', 'none')
    .attr('stroke', 'none')

  // rendering horizontal grid on xAxis
  const grid = graphArea
    .append('g')
    .attr('class', 'grid')
    .attr('transform', `translate(${paddingLeft},${paddingTop / 2})`)
    .call(showHorizontalLines({ yScale, svgWidth, showGradeLabel }))
  grid
    .selectAll('path')
    .attr('fill', 'none')
    .attr('stroke', 'none')
  grid
    .selectAll('line')
    .attr('class', 'horizontal-lines')
    .attr('stroke-width', '1')
    .attr('stroke', '#C9CFD6')
    .attr('fill', '#C9CFD6')

  // CREATE LINES
  const lineDrawInfo = getLinesInfoToDrawChart(
    behaviorScores,
    keyToBeDashed,
    sortAlphabetical,
    isInsight
  )

  const createLines = key => {
    if (disabledLines.includes(key) || !lineDrawInfo[key]) return
    const lineData = dataGraph[key]
    const { color, isDashed, opacity } = lineDrawInfo[key]
    const valueLine = d3
      .line()
      .x(item => {
        return xScale(parseXAxis(item, typeGraph))
      })
      .y(item => {
        const v = isInsight ? item.individuals : item.score
        if (v !== null) return yScale(v)
      })
      .defined(item => {
        const v = isInsight ? item.individuals : item.score
        return v !== null
      })

    // Add the value line paths.
    const line = graphArea.append('path')
    const translateX =
      parseFloat(
        xAxisRender
          .selectAll('.tick')
          .nodes()[0]
          .getAttribute('transform')
          .match(/translate\(([0-9\.]*)/)[1]
      ) + 35

    line
      .data([lineData])
      .attr('class', 'line')
      .style('opacity', opacity)
      .style('cursor', 'pointer')
      .attr('fill', 'none')
      .attr('stroke', color)
      .attr('stroke-width', '2')
      .attr('transform', `translate(${translateX + paddingTop},${paddingTop / 2 - 1} )`)
      .on('mouseenter', function() {
        d3.select(this).style('opacity', 1)
      })
      .on('mouseleave', function() {
        d3.select(this).style('opacity', opacity)
      })
      .on('click', function() {
        if (performanceFlag) {
          changeLineFocus(key)
        }
      })
      .attr('d', valueLine)

    if (!chartReady) {
      const pathLength = line.node().getTotalLength && line.node().getTotalLength()
      line
        .attr('stroke-dasharray', pathLength + ' ' + pathLength)
        // Set the intial starting position so that only the gap is shown by offesetting by the total length of the line
        .attr('stroke-dashoffset', pathLength)
        // Then the following lines transition the line so that the gap is hidden...
        .transition()
        .duration(1800)
        .ease(d3.easeSinIn)
        .attr('stroke-dashoffset', 0)
    }

    // insights: create horizontal bar
    if (isInsight) {
      const barWidth = xScale.domain().length * xScale.step()
      graphArea
        .append('rect')
        .attr('class', 'bar')
        .attr('fill', '#B8E5E3')
        .style('opacity', '0.3')
        .style('cursor', 'pointer')
        .attr('y', 50)
        .attr('width', barWidth)
        .attr('height', 54 * 1.5)
        .attr('transform', `translate(${paddingLeft}, ${151})`)
    }

    if (isDashed) {
      const dotsLength = line.node().getTotalLength && line.node().getTotalLength()

      // removes null points as we do not want to put dot there
      const filteredLineData = lineData.reduce((acc, cur) => {
        const v = isInsight ? cur.individuals : cur.score
        if (v !== null) acc.push(cur)
        return acc
      }, [])

      const dots = graphArea
        .selectAll('dot')
        .data(filteredLineData)
        .enter()
        .append('circle')
        .attr('class', 'hollow-point')
        .attr('fill', 'white')
        .attr('stroke', color)
        .attr('stroke-width', '2')
        .style('cursor', 'pointer')
        .attr('id', ({ date, score }) => getSanitizedId(key, date, score))
        .on('mouseenter', ({ date, score }) => {
          const fullInfo = filteredLineData.find(l => l.date === date)
          onMouseEnter({
            behaviourName: key,
            elementId: getSanitizedId(key, date, score),
            ...fullInfo,
            color,
            typeGraph
          })
        })
        .on('mouseleave', function() {
          d3.select(this).attr('fill', 'white')
        })
        .attr('transform', `translate(${translateX + paddingTop},${paddingTop / 2 - 1})`)
        .attr('r', 6)
        .attr('cx', function(item) {
          return xScale(parseXAxis(item, typeGraph)) - 1
        })
        .attr('cy', function(item) {
          return yScale(isInsight ? item.individuals : item.score)
        })
      if (!chartReady) {
        dots
          .attr('stroke-dasharray', dotsLength + ' ' + dotsLength)
          // Set the intial starting position so that only the gap is shown by offesetting by the total length of the line
          .attr('stroke-dashoffset', dotsLength)
          // Then the following lines transition the line so that the gap is hidden...
          .transition()
          .duration(4500)
          .ease(d3.easeSinIn)
          .attr('stroke-dashoffset', 0)
      }
    }

    if (behaviorScores?.intervention_data?.length > 0) {
      graphArea
        .selectAll('intervention-lines')
        .data(behaviorScores.intervention_data)
        .enter()
        .append('line')
        .attr('stroke', '#1CB696')
        .attr('stroke-width', '2')
        .attr('class', 'intervention-lines')
        .attr('transform', `translate(${translateX},${paddingTop / 2 - 1} )`)
        .attr('x1', d => xScale2(new Date(d.date)) + setLineMargin(new Date(d.date)))
        .attr('x2', d => xScale2(new Date(d.date)) + setLineMargin(new Date(d.date)))
        .attr('y1', yScale.range()[0])
        .attr('y2', yScale.range()[1])
        .attr('id', ({ date, rdr_name }) =>
          getSanitizedId(key, date, rdr_name.replaceAll(' ', '_'))
        )
        .style('stroke-dasharray', '5')
        .style('outline', 'none')
        .style('cursor', 'pointer')
        .on('mouseenter', ({ date, rdr_name }) => {
          const fullInfo = lineData.find(l => l.date === date)
          onMouseEnter({
            behaviourName: key,
            elementId: getSanitizedId(key, date, rdr_name.replaceAll(' ', '_')),
            ...fullInfo,
            typeGraph,
            intervention: true,
            rdr_name: rdr_name,
            date: date
          })
        })
        .on('mouseleave', function() {
          d3.select(this).attr('fill', 'white')
        })
    }
  }

  if (!campaignScores || !campaignScores.length || hideActionsTaken) {
    // create lines and dots
    Object.keys(dataGraph)
      .filter(item => item !== 'intervention_data')
      .forEach(createLines)
  } else {
    // CONFIG THE BARS
    const campaignScoresData = campaignScores.map((item, index) => ({
      ...item,
      date: formatDate({
        date: item.end_date || item.start_date,
        format: 'MMM'
      }),
      quarter: moment(item.end_date || item.start_date, 'YYYY-MM-DD').format('[Q]Q Y'),
      score: 100,
      index
    }))

    const bucketCount = {}
    const barXPosition = {}

    // create an array with month names
    const monthsArray = behaviorScores[Object.keys(behaviorScores)[0]].map(({ date }) =>
      formatDate({ date, format: 'MMM' })
    )

    // create an array with quarter names
    const quartersArray = dataGraph[Object.keys(behaviorScores)[0]]?.map(({ quarter }) => quarter)

    //total values on x-axis
    const xAxisCount = typeGraph === 'monthly' ? monthsArray.length : quartersArray.length

    // bar offset
    // extra space on both left and right
    const extraSpace = svgWidth * (typeGraph === 'monthly' ? 0.06 : 0.16)
    const xOffset = paddingLeft + extraSpace * 0.5 // padding + extra space on left

    // set bars position
    campaignScoresData.forEach(({ index, date, quarter }) => {
      const bucketIndex =
        typeGraph === 'monthly'
          ? monthsArray.findIndex(m => m === date)
          : quartersArray.findIndex(q => q === quarter)

      const bucketKey = typeGraph === 'monthly' ? date : quarter
      bucketCount[bucketKey] = bucketCount[bucketKey] ? bucketCount[bucketKey] + 1 : 1

      barXPosition[index] =
        //initial x-offset
        xOffset +
        // col position
        bucketIndex * (svgWidth / xAxisCount) +
        // bar number
        bucketCount[bucketKey] * 10
    })

    const xScaleBars = xScale.padding(0)
    const barWidth = 34
    graphArea
      .selectAll('.bar')
      .data(campaignScoresData)
      .enter()
      .append('rect')
      .attr('class', 'bar')
      .attr('fill', '#384F84')
      .style('opacity', '0.1')
      .style('cursor', 'pointer')
      .attr('transform', ({ index }) => `translate(${barXPosition[index]},${paddingTop / 2})`)
      .attr('id', d => d.campaign_type + d.id)
      .attr('x', function(d) {
        return xScaleBars(d.date)
      })
      .attr('y', function() {
        return yScale(100)
      })
      .attr('width', barWidth)
      .attr('height', svgHeight - paddingTop)
      .on('mouseenter', function(barInfo) {
        debugger
        d3.select(this).style('opacity', '0.8')
        onMouseEnter({
          isCampaignBar: true,
          elementId: barInfo.campaign_type + barInfo.id,
          ...barInfo,
          typeGraph
        })
      })
      .on('mouseout', function() {
        d3.select(this).style('opacity', '0.1')
      })

    // create lines and dots after creating bars so that score circles appear over bars
    Object.keys(dataGraph).forEach(createLines)
  }
}

export const handleOnToggleLine = (disabledLines, setDisabledLines) => key => {
  if (disabledLines.includes(key)) {
    setDisabledLines([...disabledLines.filter(e => e !== key)])
    return
  }
  setDisabledLines([...disabledLines, key])
}

const renderLegendSupport = (printing = false) => (
  behaviorsGraphInfo,
  disabledLines,
  setDisabledLines,
  campaignScores,
  hideActionsTaken,
  setHideActionsTaken,
  changeLineFocus,
  keyToBeDashed
) => {
  const keys = Object.keys(behaviorsGraphInfo)
  return (
    <React.Fragment>
      {keys.map(key => {
        const { label, color } = behaviorsGraphInfo[key]
        const toggleVisibilityTooltipBody = disabledLines.includes(key)
          ? 'Show trend line'
          : 'Hide trend line'
        const icon = disabledLines.includes(key) ? 'EyeClose' : 'Eye'
        return (
          <LegendLabel key={key}>
            {!printing && (
              <>
                <Tooltip body={toggleVisibilityTooltipBody} placement="top" readOnly size="xsm">
                  <LegendIcon
                    icon={icon}
                    onClick={() => handleOnToggleLine(disabledLines, setDisabledLines)(key)}
                  >
                    <Icon name={icon} fill="#959DA8" />
                  </LegendIcon>
                </Tooltip>
                <Tooltip body={'Focus trend line'} placement="top" readOnly size="xsm">
                  <LegendBullet color={color} />
                  <div
                    style={{ fontWeight: keyToBeDashed === key ? 'bolder' : 'normal' }}
                    onClick={() => {
                      if (!disabledLines.includes(key)) changeLineFocus(key)
                    }}
                  >
                    {label}
                  </div>
                </Tooltip>
              </>
            )}
            {printing && (
              <>
                <LegendBullet color={color} />
                {label}
              </>
            )}
          </LegendLabel>
        )
      })}

      {!printing && campaignScores && campaignScores.length > 0 && (
        <React.Fragment>
          <Separator />
          <LegendLabel onClick={() => setHideActionsTaken(!hideActionsTaken)}>
            <Tooltip
              body={hideActionsTaken ? 'Show trend line' : 'Hide trend line'}
              placement="top"
              readOnly
              size="xsm"
            >
              <LegendIcon icon={hideActionsTaken ? 'EyeClose' : 'Eye'}>
                <Icon name={hideActionsTaken ? 'EyeClose' : 'Eye'} fill="#959DA8" />
              </LegendIcon>
              <LegendBullet color="#384F84" />
            </Tooltip>
            Actions taken: <Strong>&nbsp;{campaignScores.length}</Strong>
          </LegendLabel>
        </React.Fragment>
      )}
    </React.Fragment>
  )
}

export const renderLegend = (
  behaviorsGraphInfo,
  disabledLines,
  setDisabledLines,
  campaignScores,
  hideActionsTaken,
  setHideActionsTaken,
  changeLineFocus,
  keyToBeDashed
) => {
  const printing = false
  return renderLegendSupport(printing)(
    behaviorsGraphInfo,
    disabledLines,
    setDisabledLines,
    campaignScores,
    hideActionsTaken,
    setHideActionsTaken,
    changeLineFocus,
    keyToBeDashed
  )
}

export function renderLegendForPrint(behaviorsGraphInfo, disabledLines, setDisabledLines) {
  const printing = true
  return renderLegendSupport(printing)(behaviorsGraphInfo, disabledLines, setDisabledLines)
}
