// Libraries
import React, { memo } from 'react'
import PropTypes from 'prop-types'

// Custom Hooks
import { useGraph } from './useGraph'
import { getLinesInfoToDrawChart, renderLegend } from './graph'

// Components

import Tabs from '../Tabs'
import Tab from '../Tab'

// Styles
import {
  GroupChart,
  Row,
  Column,
  AxisLabel,
  TitleLegend,
  Title,
  GraphTabsPosition,
  EmptyGraph
} from './HRSGraph.styles'

const tabs = ['Monthly', 'Quarterly']

const HRSGraph = props => {
  const {
    behaviorScores,
    campaignScores,
    showViewBy,
    showTabs,
    showExport,
    onMouseEnter,
    yAxisLabel,
    xAxisLabel,
    keyToBeDashed,
    showHeading,
    changeGraphTabsPosition,
    minMaxValue,
    sortAlphabetical,
    isInsight,
    yAxisDecimal,
    changeLineFocus,
    lineFOcus,
    performanceFlag,
    showGradeLabel,
    showLegend,
    tabsToPosition,
    containerStyle,
    emptyMessage,
    heading
  } = props
  const behaviorsGraphInfo = getLinesInfoToDrawChart(
    behaviorScores,
    keyToBeDashed,
    sortAlphabetical,
    isInsight
  )

  const {
    chartRef,
    printChartRef,
    disabledLines,
    setDisabledLines,
    hideActionsTaken,
    setHideActionsTaken,
    setTypeGraph
  } = useGraph({
    behaviorScores,
    campaignScores,
    onMouseEnter,
    showExport,
    keyToBeDashed,
    minMaxValue,
    sortAlphabetical,
    isInsight,
    yAxisDecimal,
    changeLineFocus,
    lineFOcus,
    performanceFlag,
    showGradeLabel
  })

  const handleOnChangeTypeGraph = index => setTypeGraph(index > 0 ? 'quarter' : 'monthly')

  if (!behaviorScores || !Object.keys(behaviorScores).length) {
    return <EmptyGraph>{emptyMessage}</EmptyGraph>
  }
  return (
    <div style={{ position: 'relative', ...containerStyle }}>
      {showExport && (
        <PrintWrapper ref={ref}>
          <HistoricalTrendForPrint
            printChartRef={printChartRef}
            behaviorScores={behaviorScores}
            behaviorsGraphInfo={behaviorsGraphInfo}
            disabledLines={disabledLines}
            setDisabledLines={setDisabledLines}
          />
        </PrintWrapper>
      )}
      {showHeading && (
        <Row justifyContent="space-between" alignItems="center" padding="0 8px 26px 15px">
          <Title>{heading}</Title>
          {showExport && (
            <IconLink
              iconName="Download"
              iconSize="xsm"
              fontWeight="bold"
              text="Export current graph as pdf"
              onClick={handlePrint}
            />
          )}
        </Row>
      )}
      <GraphTabsPosition top={tabsToPosition}>
        <Row
          justifyContent="flex-end"
          alignItems="center"
          padding="0 8px 26px 15px"
          className={changeGraphTabsPosition ? 'changeGraphTabsPosition' : 'filters-row'}
        >
          {showViewBy && (
            <ViewBy
              viewByState={viewByState}
              onViewByStateChange={setViewByState}
              selection={viewBySelection}
              onViewBySelectionChange={setViewBySelection}
            />
          )}
          {showTabs && (
            <Tabs onChange={handleOnChangeTypeGraph}>
              {tabs.map(tab => (
                <Tab title={`${tab}`} key={tab} />
              ))}
            </Tabs>
          )}
        </Row>
      </GraphTabsPosition>
      <Row>
        <Column>
          <AxisLabel transform="rotate(180deg)" writingMode="vertical-rl" textOrientation="mixed">
            {yAxisLabel}
          </AxisLabel>
        </Column>
        <Column width="100%">
          <GroupChart ref={chartRef} />
          <AxisLabel margin="0  0 0 auto" padding="0 0 0 0">
            {xAxisLabel}
          </AxisLabel>
        </Column>
        {showLegend && (
          <Column minWidth="187px" style={{ paddingLeft: 16 }}>
            <TitleLegend padding="0 0 18px 0">Show trend lines for:</TitleLegend>
            {renderLegend(
              behaviorsGraphInfo,
              disabledLines,
              setDisabledLines,
              campaignScores,
              hideActionsTaken,
              setHideActionsTaken,
              changeLineFocus,
              keyToBeDashed
            )}
          </Column>
        )}
      </Row>
    </div>
  )
}

HRSGraph.defaultProps = {
  campaignScores: [],
  onMouseEnter: () => null,
  onViewByChange: () => null,
  showViewBy: false,
  showTabs: false,
  showExport: false,
  graphTitle: 'Historical Trend',
  yAxisLabel: 'Score Distribution',
  xAxisLabel: 'Time',
  keyToBeDashed: 'overall',
  showHeading: true,
  changeGraphTabsPosition: false,
  minMaxValue: { min: 0, max: 100 },
  sortAlphabetical: false,
  isInsight: false,
  yAxisDecimal: false,
  performanceFlag: false,
  showGradeLabel: false,
  showLegend: false,
  tabsToPosition: -10,
  containerTopMargin: 80,
  emptyMessage: 'No data, cannot plot graph',
  heading: 'Historical Trend'
}

HRSGraph.propTypes = {
  onMouseEnter: PropTypes.func,
  onViewByChange: PropTypes.func,
  behaviorScores: PropTypes.shape({}).isRequired,
  campaignScores: PropTypes.arrayOf(PropTypes.shape({})),
  showViewBy: PropTypes.bool,
  showTabs: PropTypes.bool,
  showExport: PropTypes.bool,
  yAxisLabel: PropTypes.string,
  xAxisLabel: PropTypes.string,
  keyToBeDashed: PropTypes.string,
  showHeading: PropTypes.bool,
  changeGraphTabsPosition: PropTypes.bool,
  minMaxValue: PropTypes.object,
  sortAlphabetical: PropTypes.bool,
  isInsight: PropTypes.bool,
  yAxisDecimal: PropTypes.bool,
  showGradeLabel: PropTypes.bool,
  showLegend: PropTypes.bool,
  tabsToPosition: PropTypes.number,
  containerStyle: PropTypes.shape({}),
  emptyMessage: PropTypes.string,
  heading: PropTypes.string
}

export default memo(HRSGraph)
