// libs
import React from 'react'
import PropTypes from 'prop-types'

// components
import Header from '../Header'
import Typography from '../Typography'
import { Button } from '../Button'
import Input from '../Input'


// styles
import { HeaderView, InputFilterContent } from './HeaderTableContainer.styles'
import Layout from '../Layout'
import { Icon } from '../Icon'


/**
 * The HeaderTableContainer it's responsible by to render
 * the header of a container page
 *
 * @export function
 * @param {object} props
 * @returns {JSX.Element}
 */
export default function HeaderTableContainer(props) {
  const {
    button,
    title,
    hasSearchInput,
    titleOverSearchInput,
    onChangeSearchInput,
    searchInputPlaceholder,
    iconRight,
    value
  } = props

  /**
   * The renderSearchInput it's responsible by to render the input
   * to filter contents
   *
   * @returns {JSX.Element}
   */
  function renderSearchInput() {
    return (
      <Input
        placeholder={searchInputPlaceholder}
        width="full"
        onChange={onChangeSearchInput}
        value={value}
        iconLeft={<Icon name="Search" size="sm" />}
        iconRight={iconRight}
      />
    )
  }

  /**
   * The renderTitleOverSearchInput it's responsible by to
   * render the text over the search input
   *
   * @returns {JSX.Element}
   */
  function renderTitleOverSearchInput() {
    return (
      <Typography.Body color="300">{titleOverSearchInput}</Typography.Body>
    )
  }

  /**
   * The renderTitle it's responsible by to render
   * the main title of container
   *
   * @returns {JSX.Element}
   */
  function renderTitle() {
    return <Typography.H1>{title}</Typography.H1>
  }

  // main
  return (
    <HeaderView container alignItems="stretch">
      <Layout item md={8}>
        <Header>
          {title && renderTitle()}
          {button}
        </Header>
      </Layout>
      <InputFilterContent item md={4}>
        {titleOverSearchInput && renderTitleOverSearchInput()}
        {hasSearchInput && renderSearchInput()}
      </InputFilterContent>
    </HeaderView>
  )
}

HeaderTableContainer.defaultProps = {
  title: 'Title example',
  hasSearchInput: false,
  searchInputPlaceholder: 'Search placeholder',
  onChangeSearchInput: () => {},
  titleOverSearchInput: null,
  button: <Button>Example</Button>,
  iconRight: null,
  value: ""
}

HeaderTableContainer.propTypes = {
  title: PropTypes.string,
  hasSearchInput: PropTypes.bool,
  searchInputPlaceholder: PropTypes.string,
  onChangeSearchInput: PropTypes.func,
  titleOverSearchInput: PropTypes.string,
  button: PropTypes.node,
  value: PropTypes.string,
  iconRight: PropTypes.oneOfType([
    PropTypes.arrayOf(PropTypes.shape({
      type: PropTypes.oneOf([Icon, 'span'])
    })),
    PropTypes.shape({
      type: PropTypes.oneOf([Icon, 'span'])
    })
  ]),
}
