// Libraries
import React, { Component } from 'react'
import PropTypes from 'prop-types'
import { select } from 'd3-selection'
import styled from 'styled-components'

// Theme
import theme from 'theme'

// D3
import { line } from 'd3-shape'

// Styles
const LineGroup = styled.g`
  font-family: ${theme.text.fontFamily.default};
`

class Line extends Component {
  componentDidMount() {
    this.drawLine()
  }

  componentDidUpdate() {
    this.drawLine(true)
  }

  drawLine(update = false) {
    const {
      data, colorArr, color, gradientId, svgHeight, xScale, yScale, xKey, yKey
    } = this.props

    // Line Data
    const lineSetup = line()
      .x(d => xScale(d[xKey]))
      .y(d => yScale(d[yKey]))
    const valueLine = lineSetup(data)

    // Color Gradient Data
    const offsets = colorArr.map((c, i) => ({
      offset: `${Math.ceil(((i + 1) / colorArr.length) * 100)}%`,
      color: c
    }))

    // Resetting line for update
    if (update) {
      select(this.lineG)
        .selectAll('linearGradient')
        .remove()
      select(this.lineG)
        .selectAll('stop')
        .remove()
      select(this.lineG)
        .selectAll('path')
        .remove()
    }

    // Add the line
    select(this.lineG)
      .append('path')
      .datum(data)
      .attr('d', valueLine)
      .attr('fill', 'none')
      .attr('stroke', () => (colorArr ? `url(#${gradientId})` : color))
      .attr('stroke-width', '3px')
      .attr('class', 'line')

    // Add the linear gradient
    if (colorArr) {
      select(this.lineG)
        .append('linearGradient')
        .attr('id', gradientId)
        .attr('gradientUnits', 'userSpaceOnUse')
        .attr('x1', 0)
        .attr('y1', svgHeight)
        .attr('x2', 0)
        .attr('y2', 0)
        .selectAll('stop')
        .data(offsets)
        .enter()
        .append('stop')
        .attr('offset', d => d.offset)
        .attr('stop-color', d => d.color)
    }
  }

  render() {
    const {
      data,
      xScale,
      yScale,
      colorArr,
      translate,
      showEndValue,
      gradientId,
      color
    } = this.props

    // Endpoint Data
    const finalX = data[data.length - 1].date
    const finalY = data[data.length - 1].value
    const scaledFinalX = xScale(finalX)
    const scaledFinalY = yScale(finalY)

    return (
      <LineGroup
        ref={(el) => {
          this.lineG = el
        }}
        transform={translate}
      >
        {showEndValue && (
          <g>
            <circle fill={`url(#${gradientId})`} cx={scaledFinalX} cy={scaledFinalY} r="4" />
            <text
              x={scaledFinalX - 3}
              y={scaledFinalY - 10}
              fontSize="12px"
              fontWeight="bold"
              fill={colorArr ? `url(#${gradientId})` : color}
              className="current-value-text"
            >
              {finalY}
            </text>
          </g>
        )}
      </LineGroup>
    )
  }
}

export default Line

Line.propTypes = {
  color: PropTypes.string,
  colorArr: PropTypes.arrayOf(PropTypes.string),
  data: PropTypes.arrayOf(
    PropTypes.shape({
      date: PropTypes.object,
      value: PropTypes.number
    })
  ),
  gradientId: PropTypes.string,
  showEndValue: PropTypes.bool,
  svgHeight: PropTypes.number,
  translate: PropTypes.string,
  xKey: PropTypes.string,
  xScale: PropTypes.func.isRequired,
  yKey: PropTypes.string,
  yScale: PropTypes.func.isRequired
}

Line.defaultProps = {
  color: 'black',
  colorArr: ['black'],
  data: [{ date: new Date(), value: 0 }],
  gradientId: 'line-gradient',
  showEndValue: true,
  svgHeight: 300,
  translate: 'translate(0, 0)',
  xKey: 'date',
  yKey: 'value'
}
