// Libraries
import React, { PureComponent } from 'react'
import PropTypes from 'prop-types'

// Styles
import {
  ModalContainer,
  ModalContent,
  ModalOverlayer,
  ModalHeader,
  ModalFooter,
  ModalLeftButtons,
  ModalBreadcrumb
} from './Modal.styles'
import GlobalStyle from '../GlobalStyle'
import Header from '../Header'
import { Button } from '../Button'
import { ButtonLink } from '../ButtonLink'
import { Icon } from '../Icon'
import Link from '../Link'
import Breadcrumb from '../Breadcrumb'

class Modal extends PureComponent {
  // default values of props
  static defaultProps = {
    header: null,
    rightButtons: [],
    leftButtons: [],
    closeButton: null,
    breadcrumb: null,
    isOpen: false,
    onOpen: () => {},
    style: {}
  }

  // validating props
  static propTypes = {
    header: PropTypes.shape({ type: PropTypes.oneOf([Header]) }),
    closeButton: PropTypes.oneOfType([
      PropTypes.shape({
        type: PropTypes.oneOf([Button, ButtonLink, Link])
      })
    ]),
    breadcrumb: PropTypes.shape({
      type: PropTypes.oneOf([Breadcrumb])
    }),
    style: PropTypes.shape({}),
    rightButtons: PropTypes.oneOfType([
      PropTypes.arrayOf(PropTypes.shape({
        type: PropTypes.oneOf([Button])
      }))
    ]),
    leftButtons: PropTypes.oneOfType([
      PropTypes.arrayOf(PropTypes.shape({
        type: PropTypes.oneOf([Button])
      }))
    ]),
    onClose: PropTypes.func.isRequired,
    onOpen: PropTypes.func,
    isOpen: PropTypes.bool
  }

  constructor() {
    super()
    this.modalOverLayerRef = React.createRef()
    this.modalContainerRef = React.createRef()
  }

  /**
   * The handleOnCloseEvent it's responsible by to
   * handle when the user decide to close
   * the modal
   *
   * @memberof Modal
   * @returns {Boolean}
   */
  handleOnCloseEvent = (e) => {
    e.preventDefault()
    e.stopPropagation()
    const { onClose } = this.props
    if (onClose) onClose(this.modalOverLayerRef)
    return false
  }

  /**
   * The renderModalHeader it's responsible by
   * to render de header text of modal
   *
   * @memberof Modal
   * @returns {JSX.Element}
   */
  renderModalHeader = () => {
    const {
      header,
      closeButton
    } = this.props

    return (
      <ModalHeader>
        {header}
        {
          !closeButton && (
            <Button
              theme="ghost"
              size="default"
              onClick={e => this.handleOnCloseEvent(e)}
            >
              <Icon name="Close" />
            </Button>
          )
        }
        {closeButton}
      </ModalHeader>
    )
  }

  /**
   * The renderModalContent it's responsible by to render the
   * custom content of modal
   * @memberof Modal
   * @returns {JSX.Element}
   */
  renderModalContent = () => {
    const { children } = this.props
    return (
      <ModalContent>
        {children}
      </ModalContent>
    )
  }

  /**
   * The renderModalFooter it's responsible by to render
   * the footer of modal bottom
   *
   * @memberof Modal
   * @returns {JSX.Element}
   */
  renderModalFooter = () => {
    const { rightButtons, leftButtons, breadcrumb } = this.props

    return (
      <ModalFooter rightButtons={rightButtons}>
        {leftButtons && (
          <ModalLeftButtons>
            {leftButtons}
          </ModalLeftButtons>
        )}
        {
          breadcrumb && (
            <ModalBreadcrumb>
              {breadcrumb}
            </ModalBreadcrumb>
          )
        }
        {
          rightButtons && (
            <ModalLeftButtons>
              {rightButtons}
            </ModalLeftButtons>
          )
        }
      </ModalFooter>
    )
  }

  /**
   * The render it's a native function of react js library
   * @reference {https://reactjs.org/docs/react-component.html#render}
   * @memberof Modal
   * @returns {JSX.Element}
   */
  render = () => {
    const { isOpen, onOpen, style } = this.props
    if (!isOpen) return null

    return (
      <>
        <ModalOverlayer
          ref={(el) => { this.modalOverLayerRef = el }}
          onClick={e => this.handleOnCloseEvent(e)}
        />
        <ModalContainer
          style={style}
          ref={(el) => {
            this.modalContainerRef = el
            onOpen(this.modalOverLayerRef, this.modalContainerRef)
          }}
        >
          {this.renderModalHeader()}
          {this.renderModalContent()}
          {this.renderModalFooter()}
        </ModalContainer>
        <GlobalStyle />
      </>
    )
  }
}

export default Modal
