// Libraries
import React, { PureComponent } from 'react'
import PropTypes from 'prop-types'
import GlobalStyle from '../GlobalStyle'
import { Text } from '../Typography'
import { Icon } from '../Icon'
import Tooltip from '../Tooltip'

// Styles
import {
  RadioButtonWrapper,
  RadioInput,
  CheckMark,
  Label,
  RadioInputContainer,
  IconWrapper,
  NestedRadioButtonWrapper
} from './RadioButton.styles'

class RadioButton extends PureComponent {
  constructor(props) {
    super(props)
    this.state = {
      currentValue: props.radioSelected
    }
  }

  static getDerivedStateFromProps(nextProps, prevState) {
    const { radioSelected, controlled } = nextProps
    const { currentValue } = prevState
    return controlled && radioSelected !== currentValue
      ? {
          currentValue: radioSelected
        }
      : null
  }

  handleOnChange = (event, value) => {
    const { onChange } = this.props
    this.setState({ currentValue: value })
    if (onChange && typeof onChange === 'function') onChange(value, event)
  }

  renderOptions = () => {
    const { options, name, invalid, tooltip, controlled, disabled, bordered } = this.props
    const { currentValue } = this.state

    if (!options) return null
    return options.map(
      ({ label, value, helpText, tooltipText, nestedRadioButton, optionDisabled }) => (
        <RadioInputContainer
          key={value}
          bordered={bordered}
          className={value !== currentValue ? 'no-selected' : 'selected'}
          disabled={optionDisabled}
        >
          <Label disabled={disabled || optionDisabled}>
            <Text fontWeight="normal" color="500">
              {label}
            </Text>
            {tooltip && (
              <IconWrapper>
                <Tooltip body={tooltipText} placement="top" size="sm" theme="light-border" readOnly>
                  <Icon name="CircleExclamation" size="xsm" fill="#959DA8" />
                </Tooltip>
              </IconWrapper>
            )}
            <br />
            <Text fontWeight="normal" color="300">
              {helpText}
            </Text>
            <RadioInput
              type="radio"
              name={name}
              value={value}
              disabled={disabled || optionDisabled}
              {...(controlled
                ? {
                    checked: value === currentValue
                  }
                : {
                    defaultChecked: value === currentValue
                  })}
              onChange={event => this.handleOnChange(event, value)}
            />
            <CheckMark border={invalid ? 'error' : 'dark'} />
          </Label>
          {nestedRadioButton && value === currentValue && (
            <NestedRadioButtonWrapper>{nestedRadioButton}</NestedRadioButtonWrapper>
          )}
        </RadioInputContainer>
      )
    )
  }

  render = () => {
    const { vertical, disabled } = this.props

    return (
      <>
        <RadioButtonWrapper vertical={vertical} disabled={disabled}>
          {this.renderOptions()}
        </RadioButtonWrapper>
        <GlobalStyle />
      </>
    )
  }
}

RadioButton.propTypes = {
  onChange: PropTypes.func.isRequired,
  options: PropTypes.arrayOf(
    PropTypes.shape({
      value: PropTypes.string.isRequired,
      label: PropTypes.string.isRequired,
      helpText: PropTypes.string,
      tooltipText: PropTypes.oneOfType([PropTypes.string, PropTypes.node])
    })
  ),
  radioSelected: PropTypes.oneOfType([PropTypes.string, PropTypes.number, PropTypes.bool]),
  vertical: PropTypes.bool,
  name: PropTypes.string,
  invalid: PropTypes.bool,
  tooltip: PropTypes.bool,
  controlled: PropTypes.bool,
  disabled: PropTypes.bool,
  bordered: PropTypes.bool
}

RadioButton.defaultProps = {
  options: null,
  radioSelected: null,
  vertical: false,
  name: 'radio',
  invalid: false,
  tooltip: false,
  controlled: true,
  disabled: false,
  bordered: false
}

export default RadioButton
