
// ////////////////////
// / HELPER RESIZER ///
// ////////////////////

function getElementInfo(element) {
  const {
    width, height, left, right
  } = element.getBoundingClientRect()
  const {
    scrollWidth, offsetWidth, clientWidth, scrollLeft
  } = element

  return {
    width,
    height,
    left,
    right,
    scrollWidth,
    offsetWidth,
    clientWidth,
    scrollLeft
  }
}

function getColumns(container, query = 'div[data-type="th"]', type = 'all') {
  switch (type) {
    case 'all':
      return container.querySelectorAll(query)
    case 'single':
      return container.querySelector(query)
    default:
      return container.querySelectorAll(query)
  }
}

function setWidth(element, width, containerWidth, widthCorrectionDelta = 0) {
  let newWidth = parseInt(width, 0)
  const minWidth = 50

  newWidth = newWidth < minWidth ? minWidth : newWidth
  newWidth -= widthCorrectionDelta

  element.style.minWidth = `${newWidth}px`
  element.style.width = `${newWidth}px`
  element.style.flex = `0 0 ${newWidth}px`
  element.setAttribute('data-width', newWidth)
  element.setAttribute('data-container-width', containerWidth)
}


function resizeColumns(columns, size, containerWidth, cb) {
  columns.forEach((element) => {
    const hasResizerWidth = element.hasAttribute('data-resizer-width')
    const hasDataWidth = element.hasAttribute('data-width')
    if (hasResizerWidth) {
      const resizedWidth = element.getAttribute('data-resizer-width')
      setWidth(element, resizedWidth, containerWidth)
    }

    if (!hasDataWidth && !hasResizerWidth) {
      setWidth(element, Math.floor(containerWidth / size) - 10, containerWidth)
    }

    if (element.hasAttribute('data-container-width') && !hasResizerWidth) {
      setWidth(element, Math.floor(containerWidth / size) - 10, containerWidth)
    }

    if (element.hasAttribute('data-column-key') && cb) {
      const key = element.getAttribute('data-column-key')
      const dataWidth = parseInt(element.getAttribute('data-width'), 0)
      const resizedWidth = element.getAttribute('data-resizer-width')
      cb({ key, dataWidth, dataResizerWidth: resizedWidth })
    }
  })
}


function onMoveResizer({
  event, key, isMouseDown, tableContainer
}, cb) {
  if (isMouseDown) {
    const columns = getColumns(tableContainer, `div[data-column-key="${key}"]`)
    const { pageX } = event
    const { width: containerWidth } = getElementInfo(tableContainer)

    columns.forEach((element) => {
      const { left } = getElementInfo(element)
      const newWidth = pageX - left
      element.setAttribute('data-resizer-width', newWidth)
      setWidth(element, newWidth, containerWidth, 0)

      const dataWidth = newWidth
      const dataResizerWidth = parseInt(element.getAttribute('data-resizer-width'), 0)
      cb({ key, dataWidth, dataResizerWidth })
    })
  }
}

function onScrollBody({ event, tableHeader }) {
  const { target } = event
  const { scrollLeft } = getElementInfo(target)
  tableHeader.style.transform = `translateX(${-scrollLeft}px)`
}

export function initResizing({
  tableContainer, tableBody, tableHeader, cb
}) {
  const columnsHead = getColumns(tableContainer, 'div[data-type="th"]')
  const columnsBody = getColumns(tableContainer, 'div[data-type="td"]')
  const columnsResizer = getColumns(tableContainer, 'div[data-type="resizer"]')

  const containerWidth = () => {
    const { width } = getElementInfo(tableBody)
    return width
  }
  const size = columnsHead.length

  resizeColumns(columnsHead, size, containerWidth(), info => cb(info))
  resizeColumns(columnsBody, size, containerWidth())

  let isMouseDown = false
  let key = null

  const moveResizer = event => onMoveResizer({
    event, key, isMouseDown, tableContainer
  }, info => cb(info))

  columnsResizer.forEach((element) => {
    element.addEventListener('mousedown', () => {
      if (element.hasAttribute('data-resizer-column-key')) {
        key = element.getAttribute('data-resizer-column-key')
        isMouseDown = true
      } else {
        isMouseDown = false
      }
    })
  })

  document.addEventListener('mousemove', moveResizer)
  document.addEventListener('mouseup', () => { isMouseDown = false })

  window.addEventListener('resize', () => {
    resizeColumns(columnsHead, size, containerWidth())
    resizeColumns(columnsBody, size, containerWidth())
  })

  tableBody.addEventListener('scroll', event => onScrollBody({ event, tableHeader }))
}


// ///////////////////////
// / HELPER PAGINATION ///
// ///////////////////////

/**
   * The sortData  it's respnsible to sort
   * the data using the sort local state of the
   * table component
   *
   * @param {object} left
   * @param {object} right
   * @returns {Number}
   */
export function sortData(left, right, sort) {
  const { property, direction } = sort
  let a = left[property]
  let b = right[property]
  const isASC = direction === 'asc'
  // Check if sort value is an integer and then sort accordingly
  if (Number.isInteger(a) && Number.isInteger(b)) {
    return isASC ? (a - b) : (b - a)
  } else {
    // If value is not an integer, explicitly cast as a string to
    // protect against differing data types in data
    // If value is null, cast as empty string to protect
    // localCompare sort
    a = a ? a.toString() : ""
    b = b ? b.toString() : ""
    return isASC ? a.localeCompare(b) : b.localeCompare(a)
  }
}

/**
   * The getPaginatedAndSortedData get paginated and sort the data
   * when the table it's not manual
   *
   * @returns {Array}
   */
export function getPaginatedData(data, currentPage, limit) {
  return data.slice(currentPage * limit, (currentPage + 1) * limit)
}

/**
   * The getRange it's responsible by to get the
   * the range between number
   *
   * @param {Number} start
   * @param {Number} end
   * @returns {Array}
   */
function getRange(start, end) {
  return Array(end - start + 1)
    .fill()
    .map((v, i) => i + start)
}

/**
 * The generateArrayOfPageRangeWithDots it's responsible by to generate the
 * range pagination with dots
 *
 * @returns {Array}
 */
export function generateArrayOfPageRangeWithDots(pages, currentPage) {
  // Base cases
  if (pages === 1) return [1]
  if (pages === 2) return [1, 2]

  const delta = 3
  const length = pages
  const current = currentPage + 1

  const range = {
    start: Math.round(current - delta / 2),
    end: Math.round(current + delta / 2)
  }

  if (range.start - 1 === 1 || range.end + 1 === length) {
    range.start += 1
    range.end += 1
  }

  let pageNumbers = current > delta
    ? getRange(
      Math.min(range.start, length - delta),
      Math.min(range.end, length)
    )
    : getRange(1, Math.min(length, delta + 1))

  const withDots = (value, pair) => (
    pageNumbers.length + 1 !== length ? pair : [value]
  )

  if (pageNumbers[0] !== 1) {
    pageNumbers = withDots(1, [1, '...']).concat(pageNumbers)
  }

  if (pageNumbers[pageNumbers.length - 1] < length) {
    pageNumbers = pageNumbers.concat(withDots(length, ['...', length]))
  }
  return pageNumbers
}
