/* eslint-disable react/jsx-one-expression-per-line */
/* eslint-disable react/prop-types */
// libraries
import React, { useRef, useEffect, useState } from 'react'
import PropTypes from 'prop-types'

// components
import { Icon } from '../Icon'
import CustomSelect from '../CustomSelect'

// styles
import GlobalStyle from '../GlobalStyle'
import * as Styles from './Table.styles'
import {
  initResizing,
  getPaginatedData,
  generateArrayOfPageRangeWithDots,
  sortData
} from './Table.utils'

// static state

export default function Table(props) {
  const {
    columns: toColumns,
    data,
    pageSizeOptions,
    defaultPageSize,
    defaultPage,
    bodyHeight,
    totalCount,
    resizable,
    onChangeTable,
    onSortedChange,
    manual,
    resizingInfo,
    tableHover,
    highlightSelected,
    onClickRow,
    excludeFromClickRow,
    excludeSortColumn,
    placeholderForEmptyData,
    resetRowSelection
  } = props

  // /////////////////
  // / LOCAL STATE ///
  // /////////////////

  // static state
  let defaultSorted = toColumns.filter(({ asc }) => typeof asc === 'boolean')
  defaultSorted = defaultSorted && defaultSorted.length > 0 ? defaultSorted[0] : null
  defaultSorted = defaultSorted && {
    property: defaultSorted.key,
    asc: defaultSorted.asc
  }

  // dynamic state
  const [sort, setSort] = useState(defaultSorted)
  const [currentPage, setCurrentPage] = useState(defaultPage - 1)
  const [limit, setLimit] = useState(defaultPageSize)
  const [columns, setColumns] = useState(toColumns)
  const [currentRow, setCurrentRow] = useState(-1)
  // static state
  const count = totalCount
  const pages = Math.ceil(count / limit)

  // / refs
  const tableContainerRef = useRef(null)
  const tableBodyRef = useRef(null)
  const tableHeaderRef = useRef(null)

  // ////////////////
  // / LIFE CYCLES ///
  // ////////////////

  useEffect(() => {
    setLimit(defaultPageSize)
  }, [defaultPageSize])

  useEffect(() => {
    setColumns(toColumns)
    setCurrentPage(defaultPage - 1)
    setLimit(defaultPageSize)
  }, [toColumns])

  useEffect(() => {
    setCurrentPage(0)
  }, [totalCount])

  useEffect(() => {
    setCurrentRow(-1)
  }, [resetRowSelection])

  useEffect(() => {
    if (tableContainerRef.current && tableBodyRef.current && tableHeaderRef && resizable) {
      initResizing({
        tableContainer: tableContainerRef.current,
        tableBody: tableBodyRef.current,
        tableHeader: tableHeaderRef.current,
        cb: ({ key, dataResizerWidth, dataWidth }) => {
          resizingInfo[key] = { dataResizerWidth, dataWidth }
        }
      })
    }
  }, [tableContainerRef, tableBodyRef, tableHeaderRef, data, sort, limit, currentPage])

  // //////////////////////
  // / HANDLE FUNCTIONS ///
  // //////////////////////

  /**
   * The handleChangesAndUpdateData it's responsible by
   * handle the update of the table send the change to the parent
   * @returns {void}
   */
  function handleChangesAndUpdateData() {
    if ((pages || pages > 0) && onChangeTable) {
      const offset = currentPage * limit
      const pagination = { limit, offset }
      const params = {
        pagination,
        sort
      }
      onChangeTable(params)
    }
  }
  // Listen the changes of the table
  useEffect(() => handleChangesAndUpdateData(), [currentPage, limit, sort])

  /**
   * The handleSortHeaderTable it's responsible by to
   * set the sort from the header
   *
   * @param {string} value
   * @returns {void}
   */
  function handleSortHeaderTable(e, property, index) {
    e.preventDefault()
    const newColumns = columns.map(item => ({ ...item, asc: undefined }))
    const column = columns[index]
    const { asc: OldAscValue } = column
    const direction = !OldAscValue ? 'asc' : 'desc'
    newColumns[index] = { ...column, asc: !OldAscValue }
    setSort({ direction, property })
    setColumns(newColumns)

    if (onSortedChange) onSortedChange(property, direction)
  }

  /**
   * The handleOnPageSize it's responsible to update
   * the limit or page size of table component
   *
   * @param {number} val
   */
  function handleOnPageSize(val) {
    setLimit(val)
    setCurrentPage(0)
  }

  /**
   * The handleNavigation it's responsible by to
   * handle the changes of pagination/navigation
   * when the user click on the number or prev/next
   * buttons
   *
   * @param {DOM Event} e
   * @param {number} pageNumber
   * @returns {void}
   */
  function handleNavigation(e, pageNumber) {
    e.preventDefault()
    e.stopPropagation()
    if (pageNumber !== currentPage && (pageNumber >= 0 && pageNumber + 1 <= pages)) {
      setCurrentPage(pageNumber)
    }
  }

  // //////////////////////
  // / RENDER FUNCTIONS ///
  // //////////////////////

  /**
   * The renderData it's responsible to render the column
   * with the data for thead or tbody
   *
   * @param {object}
   * @returns {JSX.Element}
   */
  function renderData({
    val,
    key,
    type,
    asc,
    hasResizer = false,
    isHeadSort = false,
    index,
    item,
    onClickCell,
    excludeFromClickRowList
  }) {
    const dataResizerWidth = !resizingInfo[key] ? null : resizingInfo[key].dataResizerWidth

    const dataWidth = !resizingInfo[key] ? null : resizingInfo[key].dataWidth

    let defaultWidth = `calc(100% / ${columns.length})`

    if (dataResizerWidth) {
      defaultWidth = `${parseInt(dataResizerWidth, 0)}px`
    }
    if (!dataResizerWidth && dataWidth) {
      defaultWidth = `${dataWidth}px`
    }

    return (
      <Styles.TableData
        onClick={
          excludeFromClickRowList && !excludeFromClickRowList.includes(key)
            ? () => onClickCell(item)
            : null
        }
        className="table-data"
        key={Math.random()}
        defaultWidth={defaultWidth}
        data-column-key={key}
        data-type={type}
        data-resizer-width={dataResizerWidth}
      >
        {val}
        {isHeadSort && !excludeFromClickRowList.includes(key) && (
          <Styles.TableHeaderColumnSortIconContent
            onClick={e => handleSortHeaderTable(e, key, index)}
            className="table-header-sort-icon"
          >
            <Styles.TableHeaderColumnSortIcon
              name="SortTriangle"
              size="xsm"
              direction="up"
              active={asc}
            />
            <Styles.TableHeaderColumnSortIcon
              name="SortTriangle"
              size="xsm"
              direction="down"
              active={asc === false}
            />
          </Styles.TableHeaderColumnSortIconContent>
        )}
        {hasResizer && resizable && (
          <Styles.TableResizer data-resizer-column-key={key} data-type="resizer" />
        )}
      </Styles.TableData>
    )
  }

  /**
   * The renderHeader it's responsible to header of the table
   * component
   *
   * @returns {JSX.Element}
   */
  function renderHeader() {
    return (
      <Styles.TableHeader ref={tableHeaderRef} className="table-head">
        <Styles.TableRow className="table-row">
          {columns.map(({ header, key, asc, isHeadSort = true }, index) =>
            renderData({
              val: (
                <span className="span-head"
                  onClick={e => isHeadSort && !excludeSortColumn.includes(key) && handleSortHeaderTable(e, key, index)}
                  style={{ cursor: isHeadSort && !excludeSortColumn.includes(key) ? "pointer" : "auto" }}
                  title={typeof header === "string" ? header : null}
                >
                  {header}
                </span>
              ),
              key,
              type: 'th',
              hasResizer: true,
              isHeadSort,
              asc,
              index,
              excludeFromClickRowList: excludeSortColumn
            })
          )}
        </Styles.TableRow>
      </Styles.TableHeader>
    )
  }

  /**
   * The renderBody it's responsible to render the body of
   * the table when there is no data available.
   *
   * @returns {JSX.Element}
   */
  function renderEmptyData() {
    return placeholderForEmptyData
  }

  /**
   * The renderBody it's responsible to render the body of
   * the table
   *
   * @returns {JSX.Element}
   */
  function renderBody() {
    if (data.length === 0) return renderEmptyData()
    let rows = data
    if (!manual) {
      rows = getPaginatedData(data, currentPage, limit)
      rows = !sort ? rows : rows.sort((left, right) => sortData(left, right, sort))
    }

    const handleClick = (e, index) => {
      if(currentRow === index) {
        setCurrentRow(-1)
        onClickRow(e, -1)
        return // Same row clicked twice
      }
      setCurrentRow(index)
      onClickRow(e, index)
    }

    return (
      <Styles.TableBody
        ref={tableBodyRef}
        bodyHeight={bodyHeight}
        tableHover={tableHover}
        className="table-body"
      >
        {rows.map((item,index) => (
          <Styles.TableRow key={Math.random()} className="table-row" selectedRow={highlightSelected && index === currentRow}>
            {columns.map(({ key, render, title }) =>
              renderData({
                val: render
                  ? render(item[key], item)
                  : <span className="span" title={title ? title(item[key], item) : null}>
                      {item[key]}
                    </span>,
                key,
                type: 'td',
                item,
                onClickCell: (e) => handleClick(e, index),
                excludeFromClickRowList: excludeFromClickRow
              })
            )}
          </Styles.TableRow>
        ))}
      </Styles.TableBody>
    )
  }
  /**
   * The renderButtonPagination it's responsible by to render
   * the number of pagination
   *
   * @param {string} numberPage
   * @returns {JSX.Element}
   */
  function renderButtonPagination(numberPage) {
    return numberPage !== '...' ? (
      <Styles.TableNavigationPageNumber
        key={Math.random() + numberPage}
        title={`Navigation to page ${numberPage}`}
        onClick={e => handleNavigation(e, numberPage - 1)}
        disabled={currentPage === numberPage - 1}
      >
        {numberPage}
      </Styles.TableNavigationPageNumber>
    ) : (
      <span key={Math.random()} className="ellipsis">
        {numberPage}
      </span>
    )
  }

  /**
   * The renderPageNumbers it's responsible by to render the
   * number buttons of the table
   * @returns {[JSX.Element]}
   */
  function renderPageNumbers() {
    if (!pages || pages === 0) return renderButtonPagination(1)
    const numbers = []
    const pageRange = generateArrayOfPageRangeWithDots(pages, currentPage)
    for (let index = 0; index < pageRange.length; index += 1) {
      const numberPage = pageRange[index]
      numbers.push(renderButtonPagination(numberPage))
    }
    return numbers
  }

  /**
   * The renderPagination it's responsible by to render the
   * custom pagination of the table
   * @returns {[JSX.Element]}
   */
  function renderPagination() {
    const offset = currentPage * limit
    const start = count === 0 ? 0 : offset + 1
    const end = offset + limit > count ? count : offset + limit
    const nextPage = currentPage + 1
    const previousPage = currentPage - 1

    return (
      <Styles.TablePagination className="table-navigation">
        <Styles.TableNavigation>
          <Styles.TableNavigationArrow
            count={count}
            onClick={e => handleNavigation(e, previousPage)}
            disabled={currentPage === 0 || count === 0}
          >
            <Icon name="CaretLeft" />
          </Styles.TableNavigationArrow>
          <Styles.TableNavigationPageNumbersContent>
            {renderPageNumbers()}
          </Styles.TableNavigationPageNumbersContent>
          <Styles.TableNavigationArrow
            count={count}
            onClick={e => handleNavigation(e, nextPage)}
            disabled={nextPage === pages || count === 0}
          >
            <Icon name="CaretRight" />
          </Styles.TableNavigationArrow>
        </Styles.TableNavigation>
        <Styles.TableNavigationInfo>
          <CustomSelect
            disabled={count === 0}
            defaultValue={`Show ${limit} items`}
            onChange={({ index }) => handleOnPageSize(pageSizeOptions[index])}
            options={pageSizeOptions.map(item => `Show ${item} items`)}
          />
          <Styles.TableRangeInterval>
            {start} - {end} of {count.toLocaleString()}
          </Styles.TableRangeInterval>
        </Styles.TableNavigationInfo>
      </Styles.TablePagination>
    )
  }

  // main return
  return (
    <>
      <Styles.TableContainer ref={tableContainerRef}>
        {renderHeader()}
        {renderBody()}
        {renderPagination()}
      </Styles.TableContainer>
      <GlobalStyle />
    </>
  )
}

Table.defaultProps = {
  columns: [],
  data: [],
  pageSizeOptions: [25, 50, 100, 200],
  defaultPageSize: 25,
  defaultPage: 1,
  bodyHeight: 'auto',
  resizable: false,
  manual: false,
  onSortedChange: () => null,
  onChangeTable: () => null,
  resizingInfo: {},
  tableHover: false,
  highlightSelected: false,
  onClickRow: () => null,
  excludeFromClickRow: [],
  excludeSortColumn: [],
  resetRowSelection: false
}

Table.propTypes = {
  columns: PropTypes.oneOfType([PropTypes.array]),
  data: PropTypes.oneOfType([PropTypes.array]),
  pageSizeOptions: PropTypes.oneOfType([PropTypes.array]),
  defaultPageSize: PropTypes.number,
  defaultPage: PropTypes.number,
  bodyHeight: PropTypes.string,
  totalCount: PropTypes.number.isRequired,
  resizable: PropTypes.bool,
  manual: PropTypes.bool,
  onChangeTable: PropTypes.func,
  onSortedChange: PropTypes.func,
  resizingInfo: PropTypes.objectOf(
    PropTypes.shape({
      dataResizerWidth: PropTypes.number,
      dataWidth: PropTypes.number
    }),
  ),
  tableHover: PropTypes.bool,
  highlightSelected: PropTypes.bool,
  onClickRow: PropTypes.func,
  excludeFromClickRow: PropTypes.arrayOf(PropTypes.string),
  excludeSortColumn: PropTypes.arrayOf(PropTypes.string),
  resetRowSelection: PropTypes.bool
}
