// Libraries
import React, { PureComponent } from 'react'
import PropTypes from 'prop-types'

// styles
import Styles from './Tabs.styles'

// components
import Tab from '../Tab'

/**
 * The tabs it's a a tabbed interface component
 * is a container for resources associated with a tab
 *
 * @export {Class}
 * @class Tabs
 * @reference {https://www.w3.org/wiki/TabPanel}
 * @extends {PureComponent}
 */
export default class Tabs extends PureComponent {
  static defaultProps = {
    children: [],
    onChange: () => null,
    currentTabIndex: 0
  }

  static propTypes = {
    children: PropTypes.shape([PropTypes.shape({ type: Tab })]),
    onChange: PropTypes.func,
    currentTabIndex: PropTypes.number
  }

  constructor(props) {
    super(props)
    this.state = {
      selectedIndex: props.currentTabIndex
    }
  }

  componentDidUpdate(prevProps){
    if(prevProps.currentTabIndex !== this.props.currentTabIndex){
      this.setState({
        selectedIndex: this.props.currentTabIndex
      })
    }
  }

  componentDidMount = () => {
    this.setDefaultTab()
  }

  setDefaultTab = () => {
    const tabs = this.getChildrenAsArray()
    tabs.forEach(({ props: { selected } }, index) => {
      if (selected) this.setState({ selectedIndex: index })
    })
  }

  getChildrenAsArray = () => {
    const { children } = this.props
    return !Array.isArray(children) ? [children] : children
  }

  selectTab = (index) => {
    const { onChange } = this.props

    if (onChange) onChange(index)

    this.setState({
      selectedIndex: index
    })
  }

  previousTab = () => {
    const { selectedIndex } = this.state
    if (selectedIndex > 0) {
      this.selectTab(selectedIndex - 1)
    }
  }

  nextTab = () => {
    const { selectedIndex } = this.state
    const tabs = this.getChildrenAsArray()
    if (selectedIndex < tabs.length - 1) {
      this.selectTab(selectedIndex + 1)
    }
  }

  handleClick = (e, index) => {
    e.preventDefault()
    this.selectTab(index)
  }

  handleKeyup = (e) => {
    e.preventDefault()
    const direction = this.handleKeyboard(e)
    if (direction === 'right') this.nextTab()
    if (direction === 'left') this.previousTab()
  }

  /**
   * The handleKeyboard is responsible by handle
   * the triggered keys by keyCode and translate to text
   *
   * @returns {string}
   * @memberof CustomSelect
   */
  handleKeyboard = (ev) => {
    const { keyCode } = ev
    switch (keyCode) {
      case 39: // up right
        return 'right'
      case 37: // up arrow
        return 'left'
      case 38: // up arrow
        return 'up'
      case 40: // down arrow
        return 'down'
      case 13: // enter key
        return 'enter'
      case 27: // esc key
        return 'esc'
      default:
        return null
    }
  }

  render = () => {
    const {
      selectedIndex
    } = this.state
    const tabs = this.getChildrenAsArray()
    return (
      <Styles.TabsContainer>
        <Styles.TabsHeaderContainer role="tablist">
          {
              tabs.map(({ props: { title } }, index) => (
                <Styles.TabsHeader role="presentation" key={`${title}`}>
                  <Styles.TabsHeaderLink
                    id={`tab_${index}`}
                    href={`#tabpanel_${index}`}
                    role="tab"
                    aria-controls={`tab_${index}`}
                    aria-selected={index === selectedIndex}
                    tabIndex={(index === selectedIndex) ? 0 : -1}
                    onClick={e => this.handleClick(e, index)}
                    onKeyUp={e => this.handleKeyup(e)}
                    ref={(link) => { if (index === selectedIndex) this.activeLink = link }}
                  >
                    {
                      title
                    }
                  </Styles.TabsHeaderLink>
                </Styles.TabsHeader>
              ))
          }
        </Styles.TabsHeaderContainer>
        <Styles.TabsPanelContainer>
          {
            tabs.map((tab, index) => (
              React.cloneElement(tab, {
                index,
                key: index.toString(),
                isSelected: () => index === selectedIndex
              })
            ))
          }
        </Styles.TabsPanelContainer>
      </Styles.TabsContainer>
    )
  }
}
