// Libraries
import React, { PureComponent } from 'react'
import Tippy from '@tippy.js/react'
import PropTypes from 'prop-types'

// Components
import { Button } from '../Button'
import { ButtonLink } from '../ButtonLink'
import { Text } from '../Typography'
import { Icon } from '../Icon'

// Styles
import * as Styles from './Tooltip.styles'
import GlobalStyle from '../GlobalStyle'

// Utils
import { SIZES } from './utils'

class Tooltip extends PureComponent {

  // validating props
  static propTypes = {
    body: PropTypes.oneOfType([PropTypes.string, PropTypes.node]).isRequired,
    children: PropTypes.node.isRequired,
    stepNo: PropTypes.number,
    totalSteps: PropTypes.number,
    readOnly: PropTypes.bool,
    parentClass: PropTypes.string,
    visible: PropTypes.bool,
    size: PropTypes.oneOf(['default', 'sm', 'xsm']),
    primaryButton: PropTypes.string,
    secondaryButton: PropTypes.string,
    onPrimaryAction: PropTypes.func,
    onSecondaryAction: PropTypes.func,
    // react tippy props
    placement: PropTypes.string,
    trigger: PropTypes.string,
    distance: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    offset: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    theme: PropTypes.oneOf(['light', 'light-border']),
    showOnCreate: PropTypes.bool,
    hideOnClick: PropTypes.bool,
    // react tippy life-cycle hooks
    onShow: PropTypes.func,
    onShown: PropTypes.func,
    onHide: PropTypes.func,
    showCloseIcon: PropTypes.bool,
    onCloseEvent: PropTypes.func
  }

  // default values of props
  static defaultProps = {
    primaryButton: null,
    secondaryButton: null,
    stepNo: null,
    totalSteps: null,
    readOnly: false,
    parentClass: null,
    visible: null,
    size: 'default',
    onPrimaryAction: null,
    onSecondaryAction: null,
    placement: 'top',
    trigger: 'mouseenter focus',
    distance: '10',
    offset: 0,
    theme: 'light',
    showOnCreate: false,
    hideOnClick: true,
    onShow: null,
    onShown: null,
    onHide: null,
    onHidden: null,
    showCloseIcon: false,
    onCloseEvent: () => null
  }

  render() {
    const {
      body,
      children,
      stepNo,
      totalSteps,
      readOnly,
      parentClass,
      visible,
      size,
      primaryButton,
      secondaryButton,
      onPrimaryAction,
      onSecondaryAction,
      placement,
      trigger,
      distance,
      offset,
      theme,
      showOnCreate,
      hideOnClick,
      showCloseIcon,
      onCloseEvent
    } = this.props

    const showPrimaryButton = primaryButton && onPrimaryAction
    const showSecondaryButton = secondaryButton && onSecondaryAction && stepNo !== 1
    const showSteps = totalSteps
    const showFooter = showPrimaryButton || showSecondaryButton || showSteps
    const { paddingX, paddingY, fontSize } = SIZES[size]

    const Body = () => (typeof (body) === 'string' ? (
      <Styles.TooltipBodyContainer>
        <Text
          fontWeight="normal"
          style={{ fontSize: `${fontSize}px`, flex: 1 }}>
          {body}
        </Text>
        <CloseIcon />
      </Styles.TooltipBodyContainer>
    ) : (
      <Styles.TooltipBodyContainer>
        <div style={{ flex: 1 }}>
          {body}
        </div>
        <CloseIcon />
      </Styles.TooltipBodyContainer>
    )) // custom HTMl content

    const PrimaryButton = () => (showPrimaryButton ? (
      <Button onClick={e => onPrimaryAction(e)}>
        {primaryButton}
      </Button>
    ) : null)

    const SecondaryButton = () => (showSecondaryButton ? (
      <ButtonLink onClick={e => onSecondaryAction(e)} style={{ marginRight: '16px' }}>
        {secondaryButton}
      </ButtonLink>
    ) : null)

    const Steps = () => (showSteps ? (
      <Styles.TooltipStep>
        <Text color="700" fontWeight="bold">{`${stepNo} of ${totalSteps}`}</Text>
      </Styles.TooltipStep>
    ) : null)

    const Footer = () => (showFooter ? (
      <Styles.TooltipFooter>
        <PrimaryButton />
        <SecondaryButton />
        <Steps />
      </Styles.TooltipFooter>
    ) : null)

    const CloseIcon = () => (showCloseIcon ? (
      <Styles.TooltipClose>
        <Styles.CloseButton onClick={e => onCloseEvent(e)}>
          <Icon name="Close" size="sm" stroke="#959DA8" />
        </Styles.CloseButton>
      </Styles.TooltipClose>
    ) : null)

    const props = {
      content: (
        <Styles.TooltipContainer
          showPrimaryButton={showPrimaryButton}
          paddingX={paddingX}
          paddingY={paddingY}
        >
          <Body />
          <Footer />
        </Styles.TooltipContainer>
      ),
      placement,
      trigger,
      distance,
      offset,
      theme,
      interactive: !readOnly,
      showOnCreate,
      hideOnClick,
      maxWidth: (placement === 'left' || placement === 'right') ? '312px' : '320px' // 312px (wdith) + 8px (caret width)
    }
    const hooks = ['onShow', 'onShown', 'onHide', 'onHidden']
    hooks.forEach((hook) => {
    // eslint-disable-next-line react/destructuring-assignment
      if (this.props[hook]) {
      // eslint-disable-next-line react/destructuring-assignment
        props[hook] = this.props[hook]
      }
    })

    if (trigger === 'manual') props.visible = visible

    return (
      <>
        <Tippy {...props}>
          <span className={parentClass}>
            {children}
          </span>
        </Tippy>
        <GlobalStyle />
        <Styles.TippyGlobalStyle />
      </>
    )
  }
}

export default Tooltip
