/* eslint no-console: ["error", { allow: ["error", "log"] }] */
const fs = require('fs')
const path = require('path')

if (process.argv.length === 2) {
  console.error('Expected name of component')
  process.exit(0)
}

let files = null
let componentName = process.argv[2].trim()
componentName = componentName.charAt(0).toUpperCase() + componentName.slice(1)

const srcDirectory = path.resolve(__dirname, '../', './src/components')
const componentDirectory = path.resolve(srcDirectory, componentName)

const templateDirectory = path.resolve(__dirname, './templates')

console.log(`Creating new component ${componentName}`)

const fillTemplate = (templateString, templateVars) => {
  return new Function('return `' + templateString + '`;').call(templateVars)
}

const createDirectory = () => {
  return new Promise((resolve, reject) => {
    fs.mkdir(componentDirectory, (err) => {
      if (err) {
        console.error(`Something happened: ${err}`)
        reject(err)
      } else {
        resolve()
      }
    })
  })
}

const validateDirectory = () => {
  return new Promise((resolve, reject) => {
    fs.stat(componentDirectory, (err) => {
      if (!err) {
        console.error(`Component ${componentName} already exists`)
        reject(process.exit(0))
      } else {
        return resolve()
      }

      return null
    })
  })
}

const loadFile = (name) => {
  return new Promise((resolve, reject) => {
    fs.readFile(`${templateDirectory}/${name}.js`, 'utf8', (err, data) => {
      if (err) return reject(err)

      resolve(data)

      return null
    })
  })
}

const templateFile = (name) => {
  return new Promise((resolve, reject) => {
    loadFile(name)
      .then((file) => {
        const template = fillTemplate(file, {
          name: componentName
        })

        resolve(template)
      })
      .catch((err) => {
        console.log(name, err)
        return reject(err)
      })
  })
}

const writeFile = (name, val = '') => {
  return new Promise((resolve, reject) => {
    fs.writeFile(`${componentDirectory}/${name}`, val, 'utf8', (err, data) => {
      if (err) return reject(err)

      resolve(data)

      return null
    })
  })
}

const appendFile = () => {
  return new Promise((resolve, reject) => {
    fs.appendFile(`${srcDirectory}/index.js`, files[2], (err, data) => {
      if (err) return reject(err)

      resolve(data)

      return null
    })
  })
}

validateDirectory(componentName)
  .then(() => {
    return Promise.all([
      templateFile('index'),
      templateFile('styles'),
      templateFile('export'),
      templateFile('suite'),
      templateFile('story')
    ])
  })
  .then((loadedFiles) => {
    files = loadedFiles
    return createDirectory(componentName)
  })
  .then(() => {
    return Promise.all([
      writeFile('index.js', files[0]),
      writeFile(`${componentName}.styles.js`, files[1]),
      writeFile(`${componentName}.spec.js`, files[3]),
      writeFile(`${componentName}.stories.js`, files[4])
    ])
  })
  .then(() => {
    return appendFile(componentName)
  })
  .then(() => {
    console.log(`Finished created new component ${componentName}`)
  })
  .catch((err) => {
    console.error(`Issue: ${err}`)
  })
