/* eslint-disable import/no-extraneous-dependencies */
const { readdirSync, readFileSync, writeFileSync } = require('fs')
const path = require('path')
const nodeExternals = require('webpack-node-externals')
const CleanWebpackPlugin = require('clean-webpack-plugin')
const TerserPlugin = require('terser-webpack-plugin')
// prod build breaks if mini-css-extract-plugin upgraded >= 1.0.0
const MiniCssExtractPlugin = require('mini-css-extract-plugin')
const OptimizeCssAssetsPlugin = require('optimize-css-assets-webpack-plugin')

// getting each component to export separately
const entryComponentsDirectory = './src/components'
const entryComponents = readdirSync(entryComponentsDirectory, { withFileTypes: true })
  .filter(dirent => dirent.isDirectory())
  .reduce(
    (dirent, file) => Object.assign(dirent, { [file.name]: `${entryComponentsDirectory}/${file.name}/index.js` }), {}
  )

module.exports = {
  entry: {
    index: './src/index.js',
    ...entryComponents
  },
  externals: [nodeExternals()],
  optimization: {
    minimize: true,
    minimizer: [new TerserPlugin({
      parallel: true
    }), new OptimizeCssAssetsPlugin({})],
  },
  plugins: [
    new CleanWebpackPlugin(),
    new MiniCssExtractPlugin({ filename: 'assets/css/[name].css' }),
    new OptimizeCssAssetsPlugin({}),
    {
      apply: (compiler) => {
        compiler.hooks.done.tap('ReplaceUrlCss', () => {
          // replacing url fonts
          const cssPath = './dist/assets/css/index.css'
          const cssFile = readFileSync(cssPath)

          const toReaplace = '../assets/fonts/'
          const exp = new RegExp(toReaplace, 'gi')
          const newContent = cssFile.toString().replace(exp, '../fonts/')

          writeFileSync(cssPath, newContent)
          return true
        })
      }
    }
  ],
  module: {
    rules: [
      {
        test: /\.js$/,
        exclude: /node_modules|dist/,
        use: {
          loader: 'babel-loader',
        },
      },
      {
        test: /\.css/,
        use: [
          {
            loader: MiniCssExtractPlugin.loader,
            options: {
              hmr: false,
              publicPath: '../'
            },
          },
          'css-loader'
        ]
      },
      {
        test: /\.(woff|woff2|eot|ttf|svg)(\?[a-z0-9=.]+)?$/,
        use: [
          {
            loader: 'file-loader',
            options: {
              limit: false,
              name: 'assets/fonts/[name].[ext]',
              importLoaders: 1,
              modules: true
            }
          }
        ]
      },
      {
        test: /\.(jpe?g|png|gif)(\?[a-z0-9=.]+)?$/,
        use: [
          { loader: 'url-loader', options: { limit: 50000 } }
        ]
      },
    ],
  },
  output: {
    filename(file) {
      if (file.chunk.name === 'index') return 'index.js'
      return `${file.chunk.name}/index.js`
    },
    library: 'Elevate',
    libraryTarget: 'commonjs2',
    path: path.resolve(__dirname, '../', 'dist/'),
    umdNamedDefine: true,
    publicPath: 'dist'
  },
  mode: 'production',
  resolve: {
    extensions: ['.js', '.css', '.png', '.jpg', '.gif', '.eot', '.svg', '.ttf', '.woff', '.woff2', '.otf'],
  }
}
